#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ykdl.extractor import VideoExtractor
from ykdl.util.html import get_content
from ykdl.util.match import match1, matchall


from xml.dom.minidom import parseString
from ykdl.compact import compact_unquote

def location_dec(string):
    head = int(string[0])
    string = string[1:]
    rows = head
    cols = int(len(string)/rows) + 1
    
    out = ""
    full_row = len(string) % head
    for c in range(cols):
        for r in range(rows):
            if c == (cols - 1) and r >= full_row:
                continue
            if r < full_row:
                char = string[r*cols+c]
            else:
                char = string[cols*full_row+(r-full_row)*(cols-1)+c]
            out += char
    return compact_unquote(out).replace("^", "0")

class Xiami(VideoExtractor):
    name = u"Xiami (虾米音乐)"

    song_data = None

    def prepare(self):
        if not self.vid:
            self.vid = match1(self.url, 'http://www.xiami.com/song/(\d+)', 'http://www.xiami.com/song/detail/id/(\d+)')

        xml = get_content('http://www.xiami.com/song/playlist/id/{}/object_name/default/object_id/0'.format(self.vid) , charset = 'ignore')
        doc = parseString(xml)
        self.song_data = doc.getElementsByTagName("track")[0]

    def extract(self):
        self.stream_types = []
        i = self.song_data
        self.artist = i.getElementsByTagName("artist")[0].firstChild.nodeValue
        self.title = i.getElementsByTagName("songName")[0].firstChild.nodeValue
        url = location_dec(i.getElementsByTagName("location")[0].firstChild.nodeValue)
        self.stream_types.append('current')
        self.streams['current'] = {'container': 'mp3', 'video_profile': 'current', 'src' : [url], 'size': 0}


    def download_playlist(self, url, param):
        self.param = param
        if "album" in url:
            _id = match1(url, 'http://www.xiami.com/album/(\d+)')
            t = '1'
        elif "collect" in url:
            _id =match1(url, 'http://www.xiami.com/collect/(\d+)')
            t = '3'

        xml = get_content('http://www.xiami.com/song/playlist/id/{}/type/{}'.format(_id, t), charset = 'ignore')
        doc = parseString(xml)
        tracks = doc.getElementsByTagName("trackList")[0]

        #ugly code TODO
        n = 0
        for t in tracks.getElementsByTagName('track'):
           if not n % 2:
               self.song_data = t
               self.download_normal()
           n += 1

site = Xiami()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..util.html import get_content, url_size
from ..extractor import VideoExtractor
import json
import re

class Qianmo(VideoExtractor):
    name = u"阡陌 (qianmo)"

    supported_stream_types = [ 'hd' ]


    def prepare(self):

        if self.url and not self.vid:
            html = get_content(self.url)
            match = re.search(r'(.+?)var video =(.+?);', html)
            if match:
                video_info_json = json.loads(match.group(2))
                self.title = video_info_json['title']
                self.vid = video_info_json['ext_video_id']
            else:
                raise ValueError
        else:
            self.title = title or self.vid

        html = get_content('http://v.qianmo.com/player/{}'.format(self.vid))
        c = json.loads(html)
        for stream in self.supported_stream_types:
            if stream in c['seg'].keys():
                urls = []
                size = 0
                for info in c['seg'][stream][0]['url']:
                    size += url_size(info[0])
                    urls.append(info[0])
                self.streams[stream['id']] = {'container': 'mp4','video_profile': stream, 'src': urls, 'size' : size}
                self.stream_types.append(stream)

site = Qianmo()



#!/usr/bin/env python

__all__ = ['kugou_download']

from ..common import *
from json import loads
from base64 import b64decode
import re
import hashlib

def kugou_download(url, output_dir=".", merge=True, info_only=False):
    if url.lower().find("5sing")!=-1:
        #for 5sing.kugou.com
        html=get_html(url)
        ticket=r1(r'"ticket":\s*"(.*)"',html)
        j=loads(str(b64decode(ticket),encoding="utf-8"))
        url=j['file']
        title=j['songName']
        songtype, ext, size = url_info(url)
        print_info(site_info, title, songtype, size)
        if not info_only:
            download_urls([url], title, ext, size, output_dir, merge=merge)
    else:
        #for the www.kugou.com/
        return kugou_download_playlist(url, output_dir=output_dir, merge=merge, info_only=info_only)
        # raise NotImplementedError(url)       

def kugou_download_by_hash(title,hash_val,output_dir = '.', merge = True, info_only = False):
    #sample
    #url_sample:http://www.kugou.com/yy/album/single/536957.html
    #hash ->key  md5(hash+kgcloud")->key  decompile swf
    #cmd 4 for mp3 cmd 3 for m4a
    key=hashlib.new('md5',(hash_val+"kgcloud").encode("utf-8")).hexdigest()
    html=get_html("http://trackercdn.kugou.com/i/?pid=6&key=%s&acceptMp3=1&cmd=4&hash=%s"%(key,hash_val))
    j=loads(html)
    url=j['url']
    songtype, ext, size = url_info(url)
    print_info(site_info, title, songtype, size)
    if not info_only:
        download_urls([url], title, ext, size, output_dir, merge=merge)

def kugou_download_playlist(url, output_dir = '.', merge = True, info_only = False):
    html=get_html(url)
    pattern=re.compile('title="(.*?)".* data="(\w*)\|.*?"')
    pairs=pattern.findall(html)
    for title,hash_val in pairs:
        kugou_download_by_hash(title,hash_val,output_dir,merge,info_only)


site_info = "kugou.com"
download = kugou_download
# download_playlist = playlist_not_supported("kugou")
download_playlist=kugou_download_playlist



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..simpleextractor import SimpleExtractor

from ykdl.compact import build_opener, HTTPCookieProcessor, install_opener

class BoBo(SimpleExtractor):
    name = u"bobo娱乐 美女直播"

    def __init__(self):
        SimpleExtractor.__init__(self)
        self.live = True
        self.title_pattern = '<title>([^<]+)'
        self.url_pattern = 'CDNUrl: "([^"]+)'
        cookie_handler = HTTPCookieProcessor()
        opener = build_opener(cookie_handler)
        install_opener(opener)

site = BoBo()



#!/usr/bin/env python

from ..extractor import VideoExtractor
import json
from urllib import parse

class Facebook(VideoExtractor):

    name = 'Facebook'

    supported_stream_types = [ 'hd_src', 'sd_src' ]

    def prepare(self):

        html = get_content(self.url)
        self.title = match1(html, '<title id="pageTitle">(.+) \| Facebook</title>')
        s2 = parse.unquote(unicodize(match1(html, '\["params","([^"]*)"\]')))
        data = json.loads(s2)
        video_data = data["video_data"]["progressive"]
        available_stream_types = video_data.keys()
        for stream in self.supported_stream_types:
            if stream in available_stream_types:
                url = video_data[0][stream]
                _, ext, size = url_info(url)
                self.stream_types.append(stream)
                self.streams[stream] = {'container': ext, 'src': [url], 'size' : size}

site = Facebook()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..util.match import match1
from ..simpleextractor import SimpleExtractor

class Magisto(SimpleExtractor):
    name = "Magisto"

    def __init__(self):
        SimpleExtractor.__init__(self)
        self.url_pattern = '<source type="[^"]+" src="([^"]*)"'

    def get_title(self):
        title1 = match1(self.html, '<meta name="twitter:title" content="([^"]*)"')
        title2 = match1(self.html, '<meta name="twitter:description" content="([^"]*)"')
        video_hash = match1( self.url, 'http://www.magisto.com/video/([^/]+)')
        self.title = "%s %s - %s" % (title1, title2, video_hash)

site = Magisto()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ykdl.extractor import VideoExtractor
from ykdl.util.html import get_content
from ykdl.util.match import match1

import json

class Lizhi(VideoExtractor):
    name = u"Lizhi FM (荔枝电台)"
    audio_content = None
    def prepare(self):
        # url like http://www.lizhi.fm/#/549759/18864883431656710
        self.vid = match1(self.url, '/(\d+/\d+)')
        api_url = 'http://www.lizhi.fm/api/audio/'+ self.vid
        self.audio_content = json.loads(get_content(api_url))["audio"]

    def extract(self):
        self.stream_types = []
        self.title = self.audio_content["name"]
        res_url = self.audio_content["url"]
        self.stream_types.append('current')
        self.streams['current'] = {'container': 'mp3', 'video_profile': 'current', 'src' : [res_url], 'size': 0}

    def download_playlist(self, url, param):
        # like this http://www.lizhi.fm/#/31365/
        #api desc: s->start l->length band->some radio
        #http://www.lizhi.fm/api/radio_audios?s=0&l=100&band=31365
        self.param = param
        band_id = match1(url, '/(\d+)')
        #try to get a considerable large l to reduce html parsing task.
        api_url = 'http://www.lizhi.fm/api/radio_audios?s=0&l=65535&band='+band_id
        content_json = json.loads(get_content(api_url))
        for sound in content_json:
            self.audio_content = sound
            self.download_normal()

site = Lizhi()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json

from .util.download import save_url, save_urls
from .util import log
from .util.wrap import launch_player
from .util.fs import legitimize
from ykdl.util.m3u8_wrap import load_m3u8, load_live_m3u8

import sys
import datetime

class VideoExtractor():
    def __init__(self):
        self.url = None
        self.vid = None
        self.param = None
        self.password_protected = False
        self.iterable = False
        self.title = None
        self.artist = None
        self.stream_types = []
        self.streams = {}
        self.live = False


    def print_stream_info(self, stream_id):
        stream = self.streams[stream_id]
        print("    - format:        %s" % log.sprint(stream_id, log.NEGATIVE))
        if 'container' in stream:
            print("      container:     %s" % stream['container'])
        if 'video_profile' in stream:
            print("      video-profile: %s" % stream['video_profile'])
        if 'quality' in stream:
            print("      quality:       %s" % stream['quality'])
        if 'size' in stream:
            print("      size:          %s MiB (%s bytes)" % (round(stream['size'] / 1048576, 1), stream['size']))
        print("    # download-with: %s" % log.sprint("ykdl --format=%s [URL]" % stream_id, log.UNDERLINE))
        if self.param.url:
            print("Real urls:")
            if self.iterable:
                for url in self.extract_iter():
                    print("%s" % url)
            else:
                for url in stream['src']:
                    print("%s" % url)

    def jsonlize(self):
        json_dict = { 'site'   : self.name,
                      'title'  : self.title,
                      'url'    : self.url,
                      'vid'    : self.vid
                    }
        if self.param.info:
            json_dict['streams'] = self.streams
        else:
            stream_id = self.param.format or self.stream_types[0]
            json_dict['streams'] = self.streams[stream_id]
        return json_dict

    def print_info(self):
        print("site:                %s" % self.name)
        print("title:               %s" % self.title)
        print("artist:              %s" % self.artist)
        print("streams:")
        if not self.param.info:
            stream_id = self.param.format or self.stream_types[0]
            self.print_stream_info(stream_id)
        else:
            for stream_id in self.stream_types:
                self.print_stream_info(stream_id)

    def download(self, url, param):
        if isinstance(url, str) and url.startswith('http'):
            self.url = url
            self.vid= None
        else:
            self.url = None
            self.vid= url

        self.param = param
        self.stream_types = []

        self.prepare()

        if self.iterable:
            self.download_iter()
        else:
            self.download_normal()

    def prepare(self):
        pass
        #raise NotImplementedError()

    def extract(self):
        pass
        #raise NotImplementedError()

    def extract_iter(self):
        pass

    def name_suffix(self):
        if self.live:
            return datetime.datetime.now().isoformat()
        else:
            return ''

    def download_normal(self):
        self.extract()
        stream_id = self.param.format or self.stream_types[0]
        if self.param.json:
            print(json.dumps(self.jsonlize(), indent=4, sort_keys=True, ensure_ascii=False))
        else:
            self.print_info()
        if self.param.info or self.param.url:
            return
        urls = self.streams[stream_id]['src']
        if not urls:
            raise RuntimeError(self.name+ ': [Failed] Cannot extract video source from: ' + self.url if self.url else str(self.vid))
        elif self.param.player:
            launch_player(self.param.player, urls)
        else:
            name = '_'.join([legitimize(self.title), stream_id, self.name_suffix()])
            if self.streams[stream_id]['container'] == 'm3u8':
                self.streams[stream_id]['container'] = 'ts'
                if self.live:
                    urls = load_live_m3u8(urls[0])
                else:
                    urls = load_m3u8(urls[0])
            save_urls(urls, name, self.streams[stream_id]['container'])


    def download_iter(self):
        stream_id = self.param.format or self.stream_types[0]
        if self.param.json:
            print(json.dumps(self.jsonlize(), indent=4, sort_keys=True, ensure_ascii=False))
        else:
            self.print_info()
        if self.param.info or self.param.url:
            return
        i = 0
        name = '_'.join([legitimize(self.title), stream_id, self.name_suffix()])
        for url in self.extract_iter():
            if self.param.player:
                launch_player(self.param.player, [url])
            else:
                print("Download: " + self.title + " part %d" % i)
                save_url(url, name + '_%02d_.' % i + self.streams[stream_id]['container'])
                print()
                i += 1

    def prepare_list(self):
        pass

    def download_playlist(self, url, param):
        self.url = url
        video_list = self.prepare_list()
        if not video_list:
            raise NotImplementedError('Playlist is not supported for ' + self.name)
        for v in video_list[param.start:]:
            self.download(v, param)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..util.html import *
from ..util.match import *
from ..extractor import VideoExtractor

import json

class Miaopai(VideoExtractor):

    name = u'秒拍 (Miaopai)'

    def prepare(self):

        if not self.vid:
            self.vid = match1(self.url, 'http://www.miaopai.com/show/channel/([^.]+)', \
                                        'http://www.miaopai.com/show/([^.]+)', \
                                        'http://m.miaopai.com/show/channel/([^.]+)')
        content = json.loads(get_content('http://api.miaopai.com/m/v2_channel.json?fillType=259&scid={}&vend=miaopai'.format(self.vid)))

        assert content['status'] == 200, "something error!"

        content = content['result']
        self.title = content['ext']['t']
        url = content['stream']['base']
        ext = content['stream']['and']

        self.stream_types.append('current')
        self.streams['current'] = {'container': ext, 'src': [url], 'size' : 0}

    def download_playlist(self, url, param):
        html = get_content(url)
        video_list = match1(html, 'video_list=\[([^\]]+)')
        vids = matchall(video_list, ['\"([^\",]+)'])
        for vid in vids:
            self.download(vid, param)

site = Miaopai()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ykdl.util.html import get_content
from ykdl.util.match import match1
from ykdl.extractor import VideoExtractor

from ykdl.compact import compact_unquote

class NeteaseVideo(VideoExtractor):
    name = u"网易视频 (163)"
    sopported_stream_types = ['shd', 'hd', 'flv']
    stream_2_profile = {'shd': u'超清', 'hd': u'高清', 'flv': u'标清'}

    def prepare(self):

        if not self.vid:
            html = get_content(self.url)
            topiccid = match1(html, 'topicid : \"([^\"]+)', 'topicid=([^&]+)')
            vid = match1(html, 'vid : \"([^\"]+)', 'vid=([^&]+)')
            self.vid = (topiccid, vid)
        topiccid, _vid = self.vid
        code = _vid[-2:]
        video_xml = get_content('http://xml.ws.126.net/video/{}/{}/{}_{}.xml'.format(code[0], code[1], topiccid, _vid))
        self.title = compact_unquote(match1(video_xml, '<title>([^<]+)'))

        for tp in self.sopported_stream_types:
            searchcode = '<{}Url><flv>([^<]+)'.format(tp)
            url = match1(video_xml, searchcode)
            if url:
                self.stream_types.append(tp)
                self.streams[tp] = {'container': 'flv', 'video_profile': self.stream_2_profile[tp], 'src' : [url], 'size': 0}

site = NeteaseVideo()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json

from ..util.match import match1
from ..util.html import get_content, url_info
from ..extractor import VideoExtractor


class Dailymotion(VideoExtractor):
    name = "Dailymotion"

    supported_stream_types = ['720', '480', '380', '240' ]

    def prepare(self):
        html = get_content(self.url)
        info = json.loads(match1(html, r'qualities":({.+?}),"'))
        self.title = match1(html, r'"video_title"\s*:\s*"(.+?)",')

        for stream in self.supported_stream_types:
            if stream in info.keys():
                url = info[stream][0]["url"]
                _, ext, size = url_info(url)
                self.stream_types.append(stream)
                self.streams[stream] = {'container': ext, 'src': [url], 'size' : size}

site = Dailymotion()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..extractor import VideoExtractor
from ..util.match import match1, matchall
from ..util.html import get_content, get_location

import json

def get_realurl(url):
    location = get_location(url)
    if location != url:
        return location
    else:
       html = get_content(url)
       return matchall(html, ['CDATA\[([^\]]+)'])[1]

class Sina(VideoExtractor):
    name = u"新浪视频 (sina)"

    def prepare(self):

        if not self.vid:
            html = get_content(self.url)
            self.vid = match1(html, 'video_id:\'([^\']+)') or match1(self.url, '#(\d+)')

        assert self.vid, "can't get vid"

        api_url = 'http://s.video.sina.com.cn/video/h5play?video_id={}'.format(self.vid)
        info = json.loads(get_content(api_url))['data']
        self.title = info['title']
        for t in ['mp4', '3gp', 'flv']:
            if t in info['videos']:
                video_info = info['videos'][t]
                break

        for profile in video_info:
            if not profile in self.stream_types:
                v = video_info[profile]
                tp = v['type']
                url = v['file_api']+'?vid='+v['file_id']
                r_url = get_realurl(url)
                self.stream_types.append(profile)
                self.streams[profile] = {'container': tp, 'video_profile': profile, 'src': [r_url], 'size' : 0}

    def download_playlist(self, url, param):
        self.url = url

        html = get_content(self.url)
        vids = matchall(html, ['video_id: ([^,]+)'])
        print(vids)
        for v in vids:
            self.download(v, param)

site = Sina()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ykdl.util.html import get_content
from ykdl.util.match import match1
from ykdl.extractor import VideoExtractor
import json

class NeteaseLive(VideoExtractor):
    name = u"网易直播 (163)"

    def prepare(self):
        self.live = True
        if not self.vid:
            html = get_content(self.url)
            self.vid = match1(html, "anchorCcId : \'([^\']+)")
            self.title = match1(html, "<title>([^<]+)")
        else:
            self.title = self.name + str(self.vid)

        info = json.loads(get_content("http://cgi.v.cc.163.com/video_play_url/{}".format(self.vid)))

        self.stream_types.append("current")
        self.streams["current"] = {'container': 'flv', 'video_profile': "current", 'src' : [info["videourl"]], 'size': 0}

site = NeteaseLive()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..util.match import match1
from ..simpleextractor import SimpleExtractor

from ykdl.compact import compact_unquote
import re

class Metacafe(SimpleExtractor):
    name = "Metacafe"

    def __init__(self):
        SimpleExtractor.__init__(self)
        self.title_pattern = '<meta property="og:title" content="([^"]*)"'

    def l_assert(self):
        assert re.match(r'http://www.metacafe.com/watch/\w+', self.url)

    def get_url(self):
        url_raw = match1(html, '&videoURL=([^&]+)')
        self.v_url = [compact_unquote(url_raw)]

site = Metacafe()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..simpleextractor import SimpleExtractor
from ykdl.util.html import add_header

class Weibo(SimpleExtractor):
    name = u"微博秒拍 (Weibo)"

    def __init__(self):
        SimpleExtractor.__init__(self)
        add_header('User-Agent', 'Mozilla/5.0 (Linux; Android 4.4.2; Nexus 4 Build/KOT49H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/34.0.1847.114 Mobile Safari/537.36')
        self.url_pattern = '<video src="(.*?)\"\W'
        self.title_pattern = '<meta name="description" content="([\s\S]*?)\"\W'

    def l_assert(self):
        self.url = 'http://video.weibo.com/show?fid=1034:' + self.url[-32:] + '&type=mp4'

    def get_title(self):
        self.title = SimpleExtractor.get_title(self) or self.name

site = Weibo()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..util.html import *
from ..util.match import *
from ..extractor import VideoExtractor

class Vine(VideoExtractor):
    name = 'Vine'

    def prepare(self):

        if self.url and not self.vid:
            self.vid = match1(self.url, 'vine.co/v/([^/]+)')
        else:
            self.url = "https://vine.co/v/{}/".format(self.vid)

        html = get_content(self.url)

        title1 = match1(html, '<meta property="twitter:title" content="([^"]*)"')
        title2 = match1(html, '<meta property="twitter:description" content="([^"]*)"')
        self.title = "{} - {} [{}]".format(title1, title2, self.vid)

        stream = match1(html, '<meta property="twitter:player:stream" content="([^"]*)">')

        mime, ext, size = url_info(stream)

        self.stream_types.append('current')
        self.streams['current'] = {'container': ext, 'src': [stream], 'size': size }

site = Vine()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ykdl.util.html import get_content
from ykdl.util.match import match1
from ykdl.extractor import VideoExtractor
from ykdl.compact import urlparse

import json
import time
from random import random

'''
Changelog:
    1. http://tv.sohu.com/upload/swf/20150604/Main.swf
        new api
'''


class SohuBase(VideoExtractor):

    supported_stream_types = [ 'oriVid', 'superVid', 'highVid', 'norVid' ]

    realurls = { 'oriVid': [], 'superVid': [], 'highVid': [], 'norVid': [], 'relativeId': []}

    def parser_info(self, info, stream, lvid):
        if not 'allot' in info:
            return
        host = info['allot']
        prot = info['prot']
        tvid = info['tvid']
        data = info['data']
        size = sum(map(int,data['clipsBytes']))
        assert len(data['clipsURL']) == len(data['clipsBytes']) == len(data['su'])
        for new, clip, ck, in zip(data['su'], data['clipsURL'], data['ck']):
            clipURL = urlparse(clip).path
            self.realurls[stream].append('http://'+host+'/?prot=9&prod=flash&pt=1&file='+clipURL+'&new='+new +'&key='+ ck+'&vid='+str(self.vid)+'&uid='+str(int(time.time()*1000))+'&t='+str(random())+'&rb=1')
        self.streams[stream] = {'container': 'mp4', 'video_profile': stream, 'size' : size}
        self.stream_types.append(stream)

    def prepare(self):

        if self.url and not self.vid:
            html = get_content(self.url)
            self.vid = match1(html, '\/([0-9]+)\/v\.swf', '\&id=(\d+)')

        info = json.loads(get_content(self.apiurl % self.vid))
        if info['status'] == 1:
            data = info['data']
            self.title = data['tvName']
            for stream in self.supported_stream_types:
                lvid = data[stream]
                if lvid == 0 or not lvid:
                    continue
                if lvid != self.vid :
                    info = json.loads(get_content(self.apiurl % lvid))

                self.parser_info(info, stream, lvid)

    def extract(self):
        if not self.param.info:
            stream_id = self.param.format or self.stream_types[0]
            self.extract_single(stream_id)
        else:
            for stream_id in self.stream_types:
                self.extract_single(stream_id)

    def extract_single(self, stream_id):
        urls = []
        for url in self.realurls[stream_id]:
            info = json.loads(get_content(url))
            urls.append(info['url'])
        self.streams[stream_id]['src'] = urls



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..util.html import get_content, fake_headers
from ..util.match import match1, matchall
from .youkubase import YoukuBase
from .youkujs import install_acode
from ykdl.compact import HTTPSHandler, build_opener, HTTPCookieProcessor, install_opener

import time
import json
import ssl


youku_headers = fake_headers
youku_headers['Referer'] = 'v.youku.com'

class Youku(YoukuBase):
    name = u"优酷 (Youku)"

    ct = 12

    def download_playlist(self, url, param):
        self.url = url

        try:
            playlist_id = match1(self.url, 'youku\.com/playlist_show/id_([a-zA-Z0-9=]+)')
            assert playlist_id

            video_page = get_content('http://www.youku.com/playlist_show/id_%s' % playlist_id, headers=youku_headers)
            videos = matchall(video_page, ['a href="(http://v\.youku\.com/[^?"]+)'])

            for extra_page_url in matchall(video_page, ['href="(http://www\.youku\.com/playlist_show/id_%s_[^?"]+)' % playlist_id]):
                extra_page = get_content(extra_page_url, headers=youku_headers)
                videos += matchall(extra_page, ['a href="(http://v\.youku\.com/[^?"]+)'])

        except:
            video_page = get_content(url, headers=youku_headers)
            videos = videos = matchall(video_page, ['a href="(http://v\.youku\.com/[^?"]+)'])

        for video in videos:
            self.download(video, param)

    def setup(self):
        # Hot-plug cookie handler
        ssl_context = HTTPSHandler(
            context=ssl.SSLContext(ssl.PROTOCOL_TLSv1))
        cookie_handler = HTTPCookieProcessor()
        opener = build_opener(ssl_context, cookie_handler)
        opener.addheaders = [('Cookie','__ysuid=%d' % time.time())]
        install_opener(opener)



        install_acode('4', '1', 'b4et', 'boa4', 'o0b', 'poz')
        if self.url and not self.vid:
             self.vid = match1(self.url, 'youku\.com/v_show/id_([a-zA-Z0-9=]+)' ,\
                                         'player\.youku\.com/player\.php/sid/([a-zA-Z0-9=]+)/v\.swf',\
                                         'loader\.swf\?VideoIDS=([a-zA-Z0-9=]+)',\
                                         'loader\.swf\?VideoIDS=([a-zA-Z0-9=]+)',\
                                         'player\.youku\.com/embed/([a-zA-Z0-9=]+)')

        api_url = 'http://play.youku.com/play/get.json?vid=%s&ct=12' % self.vid
        api_url1 = 'http://play.youku.com/play/get.json?vid=%s&ct=10' % self.vid
        try:
            meta = json.loads(get_content(api_url, headers=youku_headers))
            meta1 = json.loads(get_content(api_url1, headers=youku_headers))
            data = meta['data']
            data1 = meta1['data']
            assert 'stream' in data1
        except:
            if 'error' in data1:
                if data1['error']['code'] == -202:
                    # Password protected
                    self.password_protected = True
                    self.password = input(log.sprint('Password: ', log.YELLOW))
                    api_url += '&pwd={}'.format(self.password)
                    api_url1 += '&pwd={}'.format(self.password)
                    meta1 = json.loads(get_content(api_url1, headers=youku_headers))
                    meta = json.loads(get_content(api_url, headers=youku_headers))
                    data1 = meta1['data']
                    data = meta['data']
                else:
                    raise AssertionError('[Failed] ' + data1['error']['note'])
            else:
                raise AssertionError('[Failed] Video not found.')

        self.title = data['video']['title']
        self.ep = data['security']['encrypt_string']
        self.ip = data['security']['ip']
        self.stream_data = data1['stream']

site = Youku()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..simpleextractor import SimpleExtractor

class JPopsuki(SimpleExtractor):
    name = "JPopsuki"
    def __init__(self):
        SimpleExtractor.__init__(self)
        self.title_pattern = '<meta name="title" content="([^"]*)"'

    def get_url(self):
        self.v_url = ["http://jpopsuki.tv{}".format(match1(self.html, '<source src="([^"]*)"'))]

site = JPopsuki()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..util.html import *
from ..util.match import *
from ..extractor import VideoExtractor

class Veoh(VideoExtractor):

    name = 'Veoh'

    def prepare(self):

        if not self.vid:
            self.vid = match1(self.url, 'http://www.veoh.com/watch/(\w+)', 'http://www.veoh.com/m/watch.php\?v=(\w+)')

        assert self.vid, 'Cannot find item ID'

        webpage_url = 'http://www.veoh.com/m/watch.php?v={}&quality=1'.format(self.vid)

        #grab download URL
        a = get_content(webpage_url)
        url = match1(a, r'<source src="(.*?)\"\W')
    
        #grab title
        self.title = match1(a, r'<meta property="og:title" content="([^"]*)"')

        type_, ext, size = url_info(url)

        self.stream_types.append('current')
        self.streams['current'] = {'container': ext, 'src': [url], 'size' : size}


site = Veoh()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..util.html import get_content, get_location
from ..util.match import matchall, match1
from ..embedextractor import EmbedExtractor

import json

"""
refer to http://open.youku.com/tools
"""
youku_embed_patterns = [ 'youku\.com/v_show/id_([a-zA-Z0-9=]+)',
                         'player\.youku\.com/player\.php/sid/([a-zA-Z0-9=]+)/v\.swf',
                         'loader\.swf\?VideoIDS=([a-zA-Z0-9=]+)',
                         'player\.youku\.com/embed/([a-zA-Z0-9=]+)',
                         'YKU.Player\(\'[a-zA-Z0-9]+\',{ client_id: \'[a-zA-Z0-9]+\', vid: \'([a-zA-Z0-9]+)\'',
                         'vid: \'([a-zA-Z0-9=]+)\','
                       ]

"""
http://www.tudou.com/programs/view/html5embed.action?type=0&amp;code=3LS_URGvl54&amp;lcode=&amp;resourceId=0_06_05_99
http://www.tudou.com/v/voahn6inu8k/&resourceId=402221676_04_02_99/v.swf
"""
tudou_embed_patterns = [ 'tudou\.com[a-zA-Z0-9\/\?=\&\.\;\#]+code=([^&]+)',
                         'tudou\.com\/[a-zA-Z]\/([^\/]+)'
                       ]

"""
refer to http://open.tudou.com/wiki/video/info
"""
tudou_api_patterns = [ ]


"""
v.qq.com
"""
qq_embed_patterns = [ 'v\.qq\.com[a-zA-Z0-9\/\?\.\;]+vid=([a-zA-Z0-9]+)',
                      'v\.qq\.com[a-zA-Z0-9\/\?\.\;]+\/([a-zA-Z0-9]+)\.html',
                      'TPout\.swf[a-zA-Z0-9=\?\&]+vid=([a-zA-Z0-9]+)'
                    ]


"""
tv.sohu.com
"""
sohu_embed_patterns = [ 'tv\.sohu\.com[a-zA-Z0-9\/\?=]+\&vid=([a-zA-Z0-9]+)\&',
                        'share\.vrs\.sohu\.com\/my\/v.swf[&+=a-zA-z0-9]+&id=([^&]+)',
                        'my\.tv\.sohu\.com\/[a-zA-Z0-9\/]+/([^\.]+)'
                      ]

"""
Youtube
"""
youtube_embed_patterns = [ 'youtu\.be/([^\"]+)',
                           'youtube\.com/embed/([^\"]+)',
                           'youtube\.com/v/([^\"]+)'
                         ]

"""
Ku6
"""
ku6_embed_url = [ '(http://v.ku6vms.com/[^\"]+)'
                     ]

"""
163
"""
netease_embed_patterns = [ 'v\.163\.com\/[0-9a-zA-Z\/\?\.]+topicid=([^&]+)&amp\;vid=([^&]+)'
                     ]

"""
iqiyi
"""
iqiyi_embed_patterns = [ 'definitionID=([^&]+)&tvId=([^&]+)'
                     ]

"""
Letv Cloud
"""
lecloud_embed_patterns = [ 'letvcloud_player_conf={"uu":"([^\"]+)","vu":"([^\"]+)"'
                     ]

"""
ifeng
"""
ifeng_embed_patterns = [ 'v\.ifeng\.com\/[a-zA-Z\=\/\?\&\.]+guid=([^\"]+)'
                     ]

"""
weibo
"""
weibo_embed_patterns = [ 'http://video.weibo.com/player/1034:(\w{32})\w*'
                     ]




class GeneralEmbed(EmbedExtractor):

    def prepare(self):
        content = get_content(self.url)
        self.title = match1(content, '<title>([^<>]+)</title>')

        vids = matchall(content, youku_embed_patterns)
        for vid in vids:
            self.video_info.append(('youku',vid))

        vids = matchall(content, tudou_embed_patterns)
        for vid in vids:
            new_url = get_location("http://tudou.com/v/"+vid)
            iid = match1(new_url, 'iid=([^&]+)')
            self.video_info.append(('tdorig',iid))

        vids = matchall(content, qq_embed_patterns)
        for vid in vids:
            self.video_info.append(('qq',vid))

        vids = matchall(content, sohu_embed_patterns)
        for vid in vids:
            self.video_info.append(('sohu.my',vid))

        vids = matchall(content, youtube_embed_patterns)
        for vid in vids:
            self.video_info.append(('youtube',vid))

        urls = matchall(content, ku6_embed_url)
        for url in urls:
            html = get_content(url)
            flashvars = matchall(html, ['vid=([^&]+)', 'style=([^&]+)', 'sn=([^&]+)'])
            data = json.loads(get_content('http://v.ku6vms.com/phpvms/player/forplayer/vid/{}/style/{}/sn/{}'.format(flashvars[0], flashvars[1],flashvars[2])))
            vid = data['ku6vid']
            self.video_info.append(('ku6',vid))
        vids = matchall(content, netease_embed_patterns)
        for v in vids:
            self.video_info.append(('netease.video', v))

        vids = matchall(content, iqiyi_embed_patterns)
        for v in vids:
            videoid, tvid = v
            self.video_info.append(('iqiyi', (tvid, videoid)))

        vids = matchall(content, lecloud_embed_patterns)
        for v in vids:
            uu, vu = v
            self.video_info.append(('letvcloud', (vu, uu)))

        vids = matchall(content, ifeng_embed_patterns)
        for v in vids:
            self.video_info.append(('ifeng', v))

        vids = matchall(content, weibo_embed_patterns)
        for v in vids:
            self.video_info.append(('weibo','http://weibo.com/p/' + v))

        tmp = []
        for v in self.video_info:
            if not v in tmp:
                tmp.append(v)
        self.video_info = tmp

site = GeneralEmbed()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json
import random
import base64, time
import sys

if sys.version_info[0] == 3:
    WR_ord = int
else:
    WR_ord = ord

from ykdl.util.html import get_content, url_info
from ykdl.util.match import match1, matchall
from ykdl.extractor import VideoExtractor

def calcTimeKey(t):
    ror = lambda val, r_bits, : ((val & (2**32-1)) >> r_bits%32) |  (val << (32-(r_bits%32)) & (2**32-1))
    return ror(ror(t,773625421%13)^773625421,773625421%17)

def decode(data):
    version = data[0:5]
    if version.lower() == b'vc_01':
        #get real m3u8
        loc2 = data[5:]
        length = len(loc2)
        loc4 = [0]*(2*length)
        for i in range(length):
            loc4[2*i] = WR_ord(loc2[i]) >> 4
            loc4[2*i+1]= WR_ord(loc2[i]) & 15;
        loc6 = loc4[len(loc4)-11:]+loc4[:len(loc4)-11]
        loc7 = [0]*length
        for i in range(length):
            loc7[i] = (loc6[2 * i] << 4) +loc6[2*i+1]
        return ''.join([chr(i) for i in loc7])
    else:
        # directly return
        return data

class Letv(VideoExtractor):
    name = u"乐视 (Letv)"

    supported_stream_types = [ '1080p', '1300', '1000', '720p', '350' ]


    def prepare(self):

        if self.vid:
            self.url = "http://www.le.com/ptv/vplay/{}.html".format(self.vid)
        html = get_content(self.url)
        if not self.vid:
            self.vid = match1(self.url, r'http://www.le.com/ptv/vplay/(\d+).html', '#record/(\d+)')
        if not self.vid:
           #self embed
           vids = matchall(html, ['vid="(\d+)"'])
           for v in vids:
               self.download(v, self.param)

        #normal process
        self.title = match1(html,r'name="irTitle" content="(.*?)"')
        info_url = 'http://api.le.com/mms/out/video/playJson?id={}&platid=1&splatid=101&format=1&tkey={}&domain=www.le.com'.format(self.vid, calcTimeKey(int(time.time())))
        r = get_content(info_url)
        info=json.loads(r)
        available_stream_id = info["playurl"]["dispatch"].keys()
        for stream in self.supported_stream_types:
            if stream in available_stream_id:
                s_url =info["playurl"]["domain"][0]+info["playurl"]["dispatch"][stream][0]
                s_url+="&ctv=pc&m3v=1&termid=1&format=1&hwtype=un&ostype=Linux&tag=le&sign=le&expect=3&tn={}&pay=0&iscpn=f9051&rateid={}".format(random.random(),stream)
                r2=get_content(s_url)
                info2=json.loads(r2)

                # hold on ! more things to do
                # to decode m3u8 (encoded)
                m3u8 = get_content(info2["location"], charset = 'ignore')
                m3u8_list = decode(m3u8)
                self.streams[stream] = {'container': 'm3u8', 'video_profile': stream, 'size' : 0}
                import tempfile
                self.streams[stream]['tmp'] = tempfile.NamedTemporaryFile(mode='w+t', suffix='.m3u8')
                self.streams[stream]['tmp'].write(m3u8_list)
                self.streams[stream]['src'] = [self.streams[stream]['tmp'].name]
                self.streams[stream]['tmp'].flush()
                self.stream_types.append(stream)

    def extract(self):
        if self.param.info:
            for stream_id in self.streams.keys():
                size = 0
                for i in self.streams[stream_id]['src']:
                    _, _, tmp = url_info(i)
                    size += tmp
                self.streams[stream_id]['size'] = size
        return
        #ignore video size in download/play mode, for preformence issue
        stream_id = self.param.format or self.stream_types[0]

        size = 0
        for i in self.streams[stream_id]['src']:
             _, _, tmp = url_info(i)
             size += tmp
             self.streams[stream_id]['size'] = size

    def download_playlist(self, url, param):
        self.url = url
        self.param = param
        html = get_content(self.url)

        vids = matchall(html, ['vid="(\d+)"'])
        for v in vids:
            self.download(v,param)

site = Letv()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..util.match import match1
from ..util.html import get_content, fake_headers
from ..extractor import VideoExtractor


class Isuntv(VideoExtractor):
    name = u"阳光卫视 (SunTV)"

    API_URL = "http://www.isuntv.com/ajaxpro/SunTv.pro_vod_playcatemp4,App_Web_playcatemp4.ascx.9f08f04f.ashx"

    def prepare(self):

        itemid = match1(self.url, r'http://www.isuntv.com/pro/ct(\d+).html')

        values = {"itemid" : itemid, "vodid": ""}
        data = str(values).replace("'", '"').encode('utf-8')

        header = fake_headers
        header['AjaxPro-Method'] = 'ToPlay'

        content = get_content(self.API_URL, data=data, headers = header)

        video_url = 'http://www.isuntv.com' + content.strip('"')

        html = get_content(self.url, charset = 'gbk')

        self.title = match1(html, '<title>([^<]+)').strip()  #get rid of \r\n s

        self.stream_types.append('current')
        self.streams['current'] = {'container': 'mp4', 'src': [video_url], 'size' : url_size(video_url)}


site = Isuntv()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..simpleextractor import SimpleExtractor
import re

class Iqilu(SimpleExtractor):
    name = u"齐鲁网 (iqilu)"

    def __init__(self):
        SimpleExtractor.__init__(self)
        self.title_pattern = '<meta name="description" content="(.*?)\"\W'
        self.url_pattern = "<input type='hidden' id='playerId' url='(.+)'"

    def l_assert(self):
        assert re.match(r'http://v.iqilu.com/\w+', self.url)

site = Iqilu()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..extractor import VideoExtractor
from ..util.html import get_content
from ..util.match import match1
import json

class Panda(VideoExtractor):
    name = u'熊猫TV (Panda)'

    live_base = "http://pl3.live.panda.tv/live_panda/{}.flv"
    api_url = "http://www.panda.tv/api_room?roomid={}"

    def prepare(self):
        self.live = True
        if not self.vid:
            self.vid = match1(self.url, 'panda.tv/(\w+)')

        content = get_content(self.api_url.format(self.vid))
        stream_data = json.loads(content)
        assert stream_data['data']['videoinfo']['status'] == '2', u"error: (⊙o⊙)主播暂时不在家，看看其他精彩直播吧！"
        room_key = stream_data['data']['videoinfo']['room_key']
        self.title = stream_data['data']['roominfo']['name']


        self.stream_types.append('current')
        self.streams['current'] = {'container': 'flv', 'video_profile': 'current', 'src' : [self.live_base.format(room_key)], 'size': float('inf')}

site = Panda()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..extractor import VideoExtractor
from .youkujs import *
from ..util.html import get_content

from ykdl.compact import urlencode

import json
from random import randint

class YoukuBase(VideoExtractor):

    def get_custom_sign(self):
        custom_api = 'https://api.youku.com/players/custom.json?client_id={}&video_id={}&vext=null&embsig=undefined&styleid=undefined'.format(self.client_id, self.vid)
        html = get_content(custom_api)
        data = json.loads(html)
        self.playsign = data['playsign']

    def get_custom_stream(self):
        custom_api = 'http://play.youku.com/partner/get.json?vid={}&ct={}&sign={}&ran={}'.format(self.vid, self.ct, self.playsign, randint(1,9999))
        html = get_content(custom_api)
        data = json.loads(html)['data']
        self.stream_data = data['stream']
        self.ep = data['security']['encrypt_string']
        self.ip = data['security']['ip']

    def prepare(self):

        self.setup()
        self.streams_parameter = {}
        for stream in self.stream_data:
            stream_id = stream['stream_type']
            if not stream_id in self.stream_types:
                self.streams_parameter[stream_id] = {
                    'fileid': stream['stream_fileid'],
                    'segs': stream['segs']
                }
                self.streams[stream_id] = {
                    'container': stream_type_to_container[stream_code_to_type[stream_id]],
                    'video_profile': stream_code_to_profiles[stream_id],
                    'size': stream['size']
                }
                self.stream_types.append(stream_id)

        self.stream_types = sorted(self.stream_types, key = supported_stream_code.index)

    def extract(self):
        if not self.param.info:
            stream_id = self.param.format or self.stream_types[0]
            self. extract_single(stream_id)
        else:
            for stream_id in self.stream_types:
                self. extract_single(stream_id)

    def extract_single(self, stream_id):
        sid, token = init(self.ep)
        segs = self.streams_parameter[stream_id]['segs']
        streamfileid = self.streams_parameter[stream_id]['fileid']
        urls = []
        no = 0
        for seg in segs:
            k = seg['key']
            assert k != -1
            fileId = getFileid(streamfileid, no)
            ep  = create_ep(sid, fileId, token)
            q = urlencode(dict(
                ctype = self.ct,
                ev    = 1,
                K     = k,
                ep    = ep,
                oip   = str(self.ip),
                token = token,
                yxon  = 1,
                myp   = 0,
                ymovie= 1,
                ts    = seg['total_milliseconds_audio'][:-3],
                hd    = stream_type_to_hd[stream_code_to_type[stream_id]],
                special = 'true',
                yyp   = 2
            ))
            nu = '%02x' % no
            u = 'http://k.youku.com/player/getFlvPath/sid/{sid}_{nu}' \
                '/st/{container}/fileid/{fileid}?{q}'.format(
                sid       = sid,
                nu        = nu,
                container = self.streams[stream_id]['container'],
                fileid    = fileId,
                q         = q
            )
            no += 1
            url = json.loads(get_content(u))[0]['server']
            urls.append(url)

        self.streams[stream_id]['src'] = urls
        if not self.streams[stream_id]['src'] and self.password_protected:
            log.e('[Failed] Wrong password.')




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
from ykdl.compact import Request, urlopen

from .html import fake_headers

def simple_hook(arg1, arg2, arg3):
    if arg3 > 0:
        percent = int(arg1 * arg2 * 100 / arg3)
        if percent > 100:
            percent = 100
        sys.stdout.write('\r %3d' % percent + '%')
        sys.stdout.flush()
    else:
        sys.stdout.write('\r' + str(round(arg1 * arg2 / 1048576, 1)) + 'MB')
        sys.stdout.flush()

def save_url(url, name, reporthook = simple_hook):
    bs = 1024*8
    size = -1
    read = 0
    blocknum = 0
    open_mode = 'wb'
    req = Request(url, headers = fake_headers)
    response = urlopen(req, None)
    if "content-length" in response.headers:
        size = int(response.headers["Content-Length"])
    if os.path.exists(name):
        filesize = os.path.getsize(name)
        if filesize == size:
            print('Skipped: file already downloaded')
            return
        elif -1 != size:
            req.add_header('Range', 'bytes=%d-' % filesize)
            blocknum = int(filesize / bs)
            response = urlopen(req, None)
            open_mode = 'ab'
    reporthook(blocknum, bs, size)
    with open(name, open_mode) as tfp:
        while True:
            block = response.read(bs)
            if not block:
                break
            read += len(block)
            tfp.write(block)
            blocknum += 1
            reporthook(blocknum, bs, size)

def save_urls(urls, name, ext):
    no = 0
    for u in urls:
        print("Download: " + name + " part %d" % no)
        n = name + '_%02d_.' % no + ext
        save_url(u, n)
        print("")
        no += 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..simpleextractor import SimpleExtractor


class Freesound(SimpleExtractor):
    name = "Freesound"

    def __init__(self):
        SimpleExtractor.__init__(self)
        self.title_pattern = '<meta property="og:title" content="([^"]*)"'
        self.url_pattern = '<meta property="og:audio" content="([^"]*)"'

site = Freesound()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .sohubase import SohuBase

class MySohu(SohuBase):
    name = u'搜狐自媒体 (MySohu)'

    apiurl = 'http://my.tv.sohu.com/play/videonew.do?vid=%s&referer=http://my.tv.sohu.com'

site = MySohu()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ykdl.util.match import match1
from ykdl.util.html import get_content
from ykdl.extractor import VideoExtractor
from ykdl.compact import urlencode, compact_bytes

import time
import json

class BaiduMusic(VideoExtractor):
    name = u'BaiduMusic (百度音乐)'


    def prepare(self):

        if not self.vid:
            self.vid = match1(self.url, 'http://music.baidu.com/song/([\d]+)')

        param = urlencode({'songIds': self.vid})

        song_data = json.loads(get_content('http://play.baidu.com/data/music/songlink', data=compact_bytes(param, 'utf-8')))['data']['songList'][0]

        self.title = song_data['songName']
        self.artist = song_data['artistName']

        self.stream_types.append('current')
        self.streams['current'] = {'container': song_data['format'], 'video_profile': 'current', 'src' : [song_data['songLink']], 'size': song_data['size']}

    def download_playlist(self, url, param):

        album_id = match1(url, 'http://music.baidu.com/album/([\d]+)')
        data = json.loads(get_content('http://play.baidu.com/data/music/box/album?albumId={}&type=album&_={}'.format(album_id, time.time())))

        print('album:		%s' % data['data']['albumName'])

        for s in data['data']['songIdList']:
            self.download(s, param)

site = BaiduMusic()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .util.html import get_content, fake_headers, url_info
from .util.match import match1
from .extractor import VideoExtractor

class SimpleExtractor(VideoExtractor):

    name = "SimpleExtractor"

    def __init__(self):
        VideoExtractor.__init__(self)

        self.html = ''

        self.title_pattern = ''

        self.url_pattern = ''

        self.artist_pattern = ''

        self.headers = fake_headers

    def get_title(self):
        if self.title_pattern:
            self.title = match1(self.html, self.title_pattern)

    def get_artist(self):
        if self.artist_pattern:
            self.artist = match1(self.html, self.artist_pattern)

    def get_url(self):
        if self.url_pattern:
            self.v_url = [match1(self.html, self.url_pattern)]

    def get_info(self):
        size=0
        ext=''
        for u in self.v_url:
            _, ext, temp = url_info(u)
            size += temp
        return ext, size

    def l_assert(self):
        pass

    def prepare(self):
        self.l_assert()
        self.html = get_content(self.url, headers=self.headers)
        self.get_title()
        self.get_artist()
        self.get_url()
        ext, size = self.get_info()
        self.stream_types.append('current')
        self.streams['current'] = {'container': ext, 'src': self.v_url, 'size' : size}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ykdl.util.match import match1
from ykdl.util.html import get_content
from ykdl.extractor import VideoExtractor
from ykdl.compact import urlencode, compact_bytes

import json

class DoubanMusic(VideoExtractor):
    name = u'Douban Music (豆瓣音乐)'

    song_info = {}

    def prepare(self):
        if not self.vid:
            self.vid = match1(self.url, 'sid=(\d+)')

        params = {
            "source" : "",
            "sids" : self.vid,
            "ck" : ""
        }
        form = urlencode(params)
        data = json.loads(get_content('https://music.douban.com/j/artist/playlist', data = compact_bytes(form, 'utf-8')))
        self.song_info = data['songs'][0]

    def extract(self):
        self.stream_types = []
        song = self.song_info
        self.title = song['title']
        self.artist = song['artist_name']
        self.stream_types.append('current')
        self.streams['current'] = {'container': 'mp3', 'video_profile': 'current', 'src' : [song['url']], 'size': 0}

    def download_playlist(self, url, param):
        self.param = param
        sids = match1(url, 'sid=([0-9,]+)')

        params = {
            "source" : "",
            "sids" : sids,
            "ck" : ""
        }
        form = urlencode(params)
        data = json.loads(get_content('https://music.douban.com/j/artist/playlist', data = compact_bytes(form, 'utf-8')))

        for s in data['songs']:
           self.song_info = s
           self.download_normal()


site = DoubanMusic()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
if sys.version_info[0] == 3:
    from urllib.request import Request, urlopen, HTTPSHandler, build_opener, HTTPCookieProcessor, install_opener, ProxyHandler
    from urllib.parse import urlencode, urlparse
    from http.client import HTTPConnection
    compact_str = str
    compact_bytes = bytes
    from urllib.parse import unquote as compact_unquote
else:
    from urllib2 import Request, urlopen, HTTPSHandler, build_opener, HTTPCookieProcessor, install_opener, ProxyHandler
    from urllib import urlencode
    from urlparse import urlparse
    from httplib import HTTPConnection
    compact_str = unicode
    def compact_bytes(string, encode):
        return string

    def compact_unquote(string, encoding = 'utf-8'):
        from urllib import unquote
        return unquote(str(string)).decode(encoding)



#!/usr/bin/env python

__all__ = ['tumblr_download']

from ..common import *

import re

def tumblr_download(url, output_dir = '.', merge = True, info_only = False):
    html = parse.unquote(get_html(url)).replace('\/', '/')
    feed = r1(r'<meta property="og:type" content="tumblr-feed:(\w+)" />', html)

    if feed == 'audio':
        real_url = r1(r'source src=\\x22([^\\]+)\\', html)
        if not real_url:
            real_url = r1(r'audio_file=([^&]+)&', html) + '?plead=please-dont-download-this-or-our-lawyers-wont-let-us-host-audio'
    elif feed == 'video':
        iframe_url = r1(r'<iframe src=\'([^\']*)\'', html)
        iframe_html = get_html(iframe_url)
        real_url = r1(r'<source src="([^"]*)"', iframe_html)
    else:
        real_url = r1(r'<source src="([^"]*)"', html)
    
    title = unescape_html(r1(r'<meta property="og:title" content="([^"]*)" />', html) or
        r1(r'<meta property="og:description" content="([^"]*)" />', html) or
        r1(r'<title>([^<\n]*)', html) or url.split("/")[4]).replace('\n', '')
    
    type, ext, size = url_info(real_url)
    
    print_info(site_info, title, type, size)
    if not info_only:
        download_urls([real_url], title, ext, size, output_dir, merge = merge)

site_info = "Tumblr.com"
download = tumblr_download
download_playlist = playlist_not_supported('tumblr')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..util.html import get_content
from ..util.match import matchall, match1
from ..extractor import VideoExtractor

from uuid import uuid4
from random import random,randint
import json
from math import floor
import hashlib

from ykdl.compact import compact_bytes

'''
Changelog:
-> http://www.iqiyi.com/common/flashplayer/20150916/MainPlayer_5_2_28_c3_3_7_4.swf
   use @fffonion 's method in #617.
   Add trace AVM(asasm) code in Iqiyi's encode function where the salt is put into the encode array and reassemble by RABCDasm(or WinRABCDasm),then use Fiddler to response modified file to replace the src file with its AutoResponder function ,set browser Fiddler proxy and play with !debug version! Flash Player ,finially get result in flashlog.txt(its location can be easily found in search engine).
   Code Like (without letters after #comment:),it just do the job : trace("{IQIYI_SALT}:"+salt_array.join(""))
   ```(Postion After getTimer)
     findpropstrict      QName(PackageNamespace(""), "trace")
     pushstring          "{IQIYI_SALT}:" #comment for you to locate the salt
     getscopeobject      1
     getslot             17 #comment: 17 is the salt slots number defined in code
     pushstring          ""
     callproperty        QName(Namespace("http://adobe.com/AS3/2006/builtin"), "join"), 1
     add
     callpropvoid        QName(PackageNamespace(""), "trace"), 1
   ```

-> http://www.iqiyi.com/common/flashplayer/20150820/MainPlayer_5_2_27_2_c3_3_7_3.swf
    some small changes in Zombie.bite function

'''

'''
com.qiyi.player.core.model.def.DefinitonEnum
bid meaning for quality
0 none
1 standard
2 high
3 super
4 suprt-high
5 fullhd
10 4k
96 topspeed

'''
def mix(tvid):
    salt = '4a1caba4b4465345366f28da7c117d20'
    tm = str(randint(2000,4000))
    sc = hashlib.new('md5', compact_bytes(salt + tm + tvid, 'utf-8')).hexdigest()
    return tm, sc, 'eknas'

def getVRSXORCode(arg1,arg2):
    loc3=arg2 %3
    if loc3 == 1:
        return arg1^121
    if loc3 == 2:
        return arg1^72
    return arg1^103


def getVrsEncodeCode(vlink):
    loc6=0
    loc2=''
    loc3=vlink.split("-")
    loc4=len(loc3)
    # loc5=loc4-1
    for i in range(loc4-1,-1,-1):
        loc6=getVRSXORCode(int(loc3[loc4-i-1],16),i)
        loc2+=chr(loc6)
    return loc2[::-1]

def getDispathKey(rid):
    tp=")(*&^flash@#$%a"  #magic from swf
    time=json.loads(get_content("http://data.video.qiyi.com/t?tn="+str(random())))["t"]
    t=str(int(floor(int(time)/(10*60.0))))
    return hashlib.new("md5",compact_bytes(t+tp+rid,"utf-8")).hexdigest()

class Iqiyi(VideoExtractor):
    name = u"爱奇艺 (Iqiyi)"

    supported_stream_types = [ '4k', 'fullhd', 'suprt-high', 'super', 'high', 'standard', 'topspeed' ]

    stream_to_bid = {  '4k': 10, 'fullhd' : 5, 'suprt-high' : 4, 'super' : 3, 'high' : 2, 'standard' :1, 'topspeed' :96}

    stream_urls = {  '4k': [] , 'fullhd' : [], 'suprt-high' : [], 'super' : [], 'high' : [], 'standard' :[], 'topspeed' :[]}

    baseurl = ''

    gen_uid = ''

    def __init__(self):
        VideoExtractor.__init__(self)
        self.iterable = True


    def getVMS(self):
        #tm ->the flash run time for md5 usage
        #um -> vip 1 normal 0
        #authkey -> for password protected video ,replace '' with your password
        #puid user.passportid may empty?
        #TODO: support password protected video
        tvid, vid = self.vid
        tm, sc, src = mix(tvid)
        uid = self.gen_uid
        vmsreq='http://cache.video.qiyi.com/vms?key=fvip&src=1702633101b340d8917a69cf8a4b8c7' +\
                "&tvId="+tvid+"&vid="+vid+"&vinfo=1&tm="+tm+\
                "&enc="+sc+\
                "&qyid="+uid+"&tn="+str(random()) +"&um=1" +\
                "&authkey="+hashlib.new('md5',compact_bytes(hashlib.new('md5', b'').hexdigest()+str(tm)+tvid,'utf-8')).hexdigest()
        return json.loads(get_content(vmsreq))



    def prepare(self):

        if self.url and not self.vid:
            vid = matchall(self.url, ['curid=([^_]+)_([\w]+)'])
            if vid:
                self.vid = vid[0]

        if self.url and not self.vid:
            html = get_content(self.url)
            tvid = match1(html, 'data-player-tvid="([^"]+)"') or match1(self.url, 'tvid=([^&]+)')
            videoid = match1(html, 'data-player-videoid="([^"]+)"') or match1(self.url, 'vid=([^&]+)')
            self.vid = (tvid, videoid)

        self.gen_uid=uuid4().hex
        info = self.getVMS()

        assert info["code"] == "A000000", "[error] outdated iQIYI key\nis your ykdl up-to-date?"


        self.title = info["data"]["vi"]["vn"]

        # data.vp = json.data.vp
        #  data.vi = json.data.vi
        #  data.f4v = json.data.f4v
        # if movieIsMember data.vp = json.data.np

        #for highest qualities
        #for http://www.iqiyi.com/v_19rrmmz5yw.html  not vp -> np
        assert info["data"]['vp']["tkl"], "[Error] Do not support for iQIYI VIP video."

        vs = info["data"]["vp"]["tkl"][0]["vs"]
        self.baseurl=info["data"]["vp"]["du"].split("/")

        for stream in self.supported_stream_types:
            for i in vs:
                if self.stream_to_bid[stream] == i['bid']:
                    video_links=i["fs"] #now in i["flvs"] not in i["fs"]
                    if not i["fs"][0]["l"].startswith("/"):
                        tmp = getVrsEncodeCode(i["fs"][0]["l"])
                        if tmp.endswith('mp4'):
                             video_links = i["flvs"]
                    self.stream_urls[stream] = video_links
                    size = 0
                    for l in video_links:
                        size += l['b']
                    self.streams[stream] = {'container': 'f4v', 'video_profile': stream, 'size' : size}
                    self.stream_types.append(stream)
                    break

    def extract_iter(self):
        stream_id = self.param.format or self.stream_types[0]

        urls=[]
        for i in self.stream_urls[stream_id]:
            vlink=i["l"]
            if not vlink.startswith("/"):
                #vlink is encode
                vlink=getVrsEncodeCode(vlink)
            key=getDispathKey(vlink.split("/")[-1].split(".")[0])
            baseurl = [x for x in self.baseurl]
            baseurl.insert(-1,key)
            url="/".join(baseurl)+vlink+'?su='+self.gen_uid+'&qyid='+uuid4().hex+'&client=&z=&bt=&ct=&tn='+str(randint(10000,20000))
            yield json.loads(get_content(url))["l"]

    def download_playlist(self, url, param):
        self.url = url

        html = get_content(self.url)

        vids = matchall(html, ['data-tvid=\"([^\"]+)\" data-vid=\"([^\"]+)\"'])
        for v in vids:
            self.download(v, param)


site = Iqiyi()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..util.html import get_content, url_info
from ..extractor import VideoExtractor
import json

class Ted(VideoExtractor):
    name = "Ted"

    supported_stream_types = ['high', 'medium', 'low']

    def prepare(self):
        html = get_content(self.url)
        metadata = json.loads(match1(html, r'({"talks"(.*)})\)'))
        self.title = metadata['talks'][0]['title']
        nativeDownloads = metadata['talks'][0]['nativeDownloads']
        for quality in self.supported_stream_types:
            if quality in nativeDownloads:
                url = nativeDownloads[quality]
                _, ext, size = url_info(url)
                self.streams[quality] = {'container': ext, 'video_profile': quality, 'src': [url], 'size' : size}
                self.stream_types.append(quality)

site = Ted()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..util.html import *
from ..util.match import *
from ykdl.util.m3u8_wrap import load_m3u8_playlist
from ..extractor import VideoExtractor
import re

class Zhanqi(VideoExtractor):
    name = u'战旗 (zhanqi)'

    live = False

    live_base = "http://dlhls.cdn.zhanqi.tv/zqlive/"
    vod_base = "http://dlvod.cdn.zhanqi.tv"
    def prepare(self):

        html = get_content(self.url)
        video_type = match1(html, 'VideoType":"([^"]+)"')
        if video_type == 'LIVE':
            self.live = True
        elif not video_type == 'VOD':
            NotImplementedError('Unknown_video_type')

        self.title = match1(html, '<title>([^<]+)')
        if self.live:
            rtmp_id = match1(html, 'videoId":"([^"]+)"').replace('\\/','/')
            real_url = self.live_base+'/'+rtmp_id+'.m3u8'
            self.stream_types, self.streams = load_m3u8_playlist(real_url)
        else:
            vod_m3u8_request = self.vod_base + match1(html, 'VideoID":"([^"]+)').replace('\\/','/')
            vod_m3u8 = get_content(vod_m3u8_request)
            part_url = re.findall(r'(/[^#]+)\.ts',vod_m3u8)
            real_url = []
            for i in part_url:
                i = self.vod_base + i + ".ts"
                real_url.append(i)
            type_ = ''
            size = 0
            for url in real_url:
                _, type_, temp = url_info(url)
                size += temp or 0
            self.stream_types.append('current')
            self.streams['current'] = {'container': 'flv', 'video_profile': 'current', 'src' : real_url, 'size': size}

site = Zhanqi()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

def main(**kwargs):
    """Main entry point.
    ykdl (legacy)
    """
    from .common import main
    try:
        main()
    except KeyboardInterrupt:
        print('\nInterrupted by Ctrl-C')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..extractor import VideoExtractor
from ..util.html import get_content, add_header
from ..util.match import match1

import json
from xml.dom.minidom import parseString

class TDorig(VideoExtractor):
    name = u"土豆原创 (tudou)"

    def prepare(self):

        if not self.title:
            self.title = self.name + "-" + self.vid
        add_header('User-Agent', '')

        data = json.loads(get_content('http://www.tudou.com/outplay/goto/getItemSegs.action?iid=%s' % self.vid))

        temp = max([data[i] for i in data if 'size' in data[i][0]], key=lambda x:sum([part['size'] for part in x]))
        vids, size = [t["k"] for t in temp], sum([t["size"] for t in temp])

        urls = []
        for vid in vids:
            for i in parseString(get_content('http://ct.v2.tudou.com/f?id=%s' % vid)).getElementsByTagName('f'):
                urls.append(i.firstChild.nodeValue.strip())

        ext = match1(urls[0], 'http://[\w.]*/(\w+)/[\w.]*')
        self.stream_types.append('current')
        self.streams['current'] = {'container': ext, 'video_profile': 'current', 'src' : urls, 'size': size}

site = TDorig()



#!/usr/bin/env python

__all__ = ['twitter_download']

from ..common import *

def twitter_download(url, output_dir='.', merge=True, info_only=False):
    html = get_html(url)
    screen_name = r1(r'data-screen-name="([^"]*)"', html)
    item_id = r1(r'data-item-id="([^"]*)"', html)
    title = "{} [{}]".format(screen_name, item_id)
    icards = r1(r'data-src="([^"]*)"', html)
    if icards:
        html = get_html("https://twitter.com" + icards)
        data = json.loads(unescape_html(r1(r'data-player-config="([^"]*)"', html)))
        source = data['playlist'][0]['source']
    else:
        source = r1(r'<source video-src="([^"]*)"', html)
    mime, ext, size = url_info(source)

    print_info(site_info, title, mime, size)
    if not info_only:
        download_urls([source], title, ext, size, output_dir, merge=merge)

site_info = "Twitter.com"
download = twitter_download
download_playlist = playlist_not_supported('twitter')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..extractor import VideoExtractor
from ..util.html import get_content, add_header
from ..util.match import match1
from ..util import log

import json

class YinYueTai(VideoExtractor):
    name = u'YinYueTai (音乐台)'
    supported_stream_types = ['sh', 'he', 'hd', 'hc' ]
    def prepare(self):

        if not self.vid:
            self.vid = match1(self.url, 'http://\w+.yinyuetai.com/video/(\d+)')

        data = json.loads(get_content('http://ext.yinyuetai.com/main/get-h-mv-info?json=true&videoId={}'.format(self.vid)))

        if data['error']:
            log.e('some error happens')

        video_data = data['videoInfo']['coreVideoInfo']

        self.title = video_data['videoName']

        for s in video_data['videoUrlModels']:
            self.stream_types.append(s['qualityLevel'])
            self.streams[s['qualityLevel']] = {'container': 'flv', 'video_profile': s['qualityLevelName'], 'src' : [s['videoUrl']], 'size': s['fileSize']}

        self.stream_types = sorted(self.stream_types, key = self.supported_stream_types.index)

    def download_playlist(self, url, param):

        playlist_id = match1(url, 'http://\w+.yinyuetai.com/playlist/(\d+)')

        playlist_data = json.loads(get_content('http://m.yinyuetai.com/mv/get-simple-playlist-info?playlistId={}'.format(playlist_id)))

        videos = playlist_data['playlistInfo']['videos']
        # TODO
        # I should directly use playlist data instead to request by vid... to be update
        for v in videos:
            vid = v['playListDetail']['videoId']
            self.download(vid, param)

site = YinYueTai()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..util.html import get_content, url_info
from ..util.match import match1
from ..extractor import VideoExtractor

import json
import re

class Ku6(VideoExtractor):
    name = u"酷6 (Ku6)"

    def prepare(self):
        if self.url and not self.vid:
            self.vid = match1(self.url, 'http://v.ku6.com/special/show_\d+/(.*)\.html',
            'http://v.ku6.com/show/(.*)\.html',
            'http://my.ku6.com/watch\?.*v=(.*).*')

        data = json.loads(get_content('http://v.ku6.com/fetchVideo4Player/%s.html' % self.vid))['data']
        self.title = data['t']
        f = data['f']


        urls = f.split(',')
        ext = re.sub(r'.*\.', '', urls[0])
        assert ext in ('flv', 'mp4', 'f4v'), ext
        ext = {'f4v': 'flv'}.get(ext, ext)
        size = 0
        for url in urls:
            _, _, temp = url_info(url)
            size += temp

        self.streams['current'] = {'container': ext, 'src': urls, 'size' : size}
        self.stream_types.append('current')

site = Ku6()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re

def get_extractor(url):
    if re.search("live.qq", url):
        from . import live as s
    elif re.search("v.qq", url):
        from . import video as s
    return s.site



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json
import time
import datetime
import platform


from ykdl.util.html import get_content, url_info
from ykdl.util.match import match1, matchall
from ykdl.extractor import VideoExtractor


class LeLive(VideoExtractor):
    name = u"Le Live (乐视轮播)"

    supported_stream_types = ['flv_1080p3m', 'flv_1080p', 'flv_1300', 'flv_1000', 'flv_720p', 'flv_350']

    def prepare(self):
        self.live = True
        if not self.vid:
            self.vid = match1(self.url, 'channel=([\d]+)')

        live_data = json.loads(get_content('http://api.live.letv.com/v1/channel/letv/100/1001/{}'.format(self.vid)))['data']

        self.title = self.name + " " + live_data['channelName']

        stream_data = live_data['streams']

        for s in stream_data:
            self.stream_types.append(s['rateType'])
            self.streams[s['rateType']] = {'container': 'm3u8', 'video_profile': s['rateType'], 'size' : float('inf'), 'streamUrl' : s['streamUrl']}

        self.stream_types = sorted(self.stream_types, key = self.supported_stream_types.index)

    def extract_single(self, stream_id):

        date = datetime.datetime.now()

        streamUrl = self.streams[stream_id]['streamUrl'] + '&format=1&expect=2&termid=1&hwtype=un&platid=10&splatid=1001&playid=1sign=live_web&&ostype={}&p1=1&p2=10&p3=-&vkit={}&station={}&tm={}'.format(platform.platform(), date.strftime("%Y%m%d"), self.vid, int(time.time()))

        data = json.loads(get_content(streamUrl))

        self.streams[stream_id]['src'] = [data['location']]

    def extract(self):
        if not self.param.info:
            stream_id = self.param.format or self.stream_types[0]
            self.extract_single(stream_id)
        else:
            for stream_id in self.stream_types:
                self.extract_single(stream_id)

site = LeLive()
        



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from argparse import ArgumentParser

def arg_parser():
    parser = ArgumentParser(description="YouKuDownLoader(ykdl), a video downloader. Forked form you-get 0.3.34@soimort")
    parser.add_argument('-l', '--playlist', action='store_true', default=False, help="Download as a playlist.")
    parser.add_argument('-i', '--info', action='store_true', default=False, help="Display the information of videos without downloading.")
    parser.add_argument('-u', '--url', action='store_true', default=False, help="Display the real URLs of videos without downloading.")
    parser.add_argument('-j', '--json', action='store_true', default=False, help="Display info in json format.")
    parser.add_argument('-F', '--format',  help="Video format code.")
    parser.add_argument('-o', '--output-dir', default='.', help="Set the output directory for downloaded videos.")
    parser.add_argument('-p', '--player', help="Directly play the video with PLAYER like vlc/smplayer")
    parser.add_argument('-s', '--start', type=int, default=0, help="start from INDEX to play/download playlist")
    parser.add_argument('--proxy', type=str, default='', help="set proxy HOST:PORT for http(s) transfer")
    parser.add_argument('-t', '--timeout', type=int, default=60, help="set socket timeout seconds, default 60s")
    parser.add_argument('video_urls', type=str, nargs='+', help="video urls")
    args = parser.parse_args()
    return args

if __name__ == '__main__':
    print(arg_parser())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
from ykdl.compact import Request, urlopen

from .match import match1

fake_headers = {
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
    'Accept-Encoding': 'gzip, deflate',
    'Accept-Language': 'zh-CN,zh;q=0.8,en-US;q=0.5,en;q=0.3',
    'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64; rv:38.0) Gecko/20100101 Firefox/38.0 Iceweasel/38.2.1'
}

def add_header(key, value):
    global fake_headers
    fake_headers[key] = value

def unicodize(text):
    return re.sub(r'\\u([0-9A-Fa-f][0-9A-Fa-f][0-9A-Fa-f][0-9A-Fa-f])', lambda x: chr(int(x.group(0)[2:], 16)), text)

def ungzip(data):
    """Decompresses data for Content-Encoding: gzip.
    """
    from io import BytesIO
    import gzip
    buffer = BytesIO(data)
    f = gzip.GzipFile(fileobj=buffer)
    return f.read()

def undeflate(data):
    """Decompresses data for Content-Encoding: deflate.
    (the zlib compression is used.)
    """
    import zlib
    decompressobj = zlib.decompressobj(-zlib.MAX_WBITS)
    return decompressobj.decompress(data)+decompressobj.flush()

def get_location(url, headers = fake_headers):
    response = urlopen(Request(url, headers = fake_headers))
    # urllib will follow redirections and it's too much code to tell urllib
    # not to do that
    return response.geturl()

def get_content(url, headers=fake_headers, data=None, charset = None):
    """Gets the content of a URL via sending a HTTP GET request.

    Args:
        url: A URL.
        headers: Request headers used by the client.
        decoded: Whether decode the response body using UTF-8 or the charset specified in Content-Type.

    Returns:
        The content as a string.
    """

    req = Request(url, headers=headers, data=data)
    #if cookies_txt:
    #    cookies_txt.add_cookie_header(req)
    #    req.headers.update(req.unredirected_hdrs)
    response = urlopen(req)
    data = response.read()

    # Handle HTTP compression for gzip and deflate (zlib)
    resheader = response.info()
    if 'Content-Encoding' in resheader:
        content_encoding = resheader['Content-Encoding']
    else:
        content_encoding = None
    if content_encoding == 'gzip':
        data = ungzip(data)
    elif content_encoding == 'deflate':
        data = undeflate(data)

    if charset == 'ignore':
        return data

    # Decode the response body
    if charset is None:
        charset = match1(resheader['Content-Type'], r'charset=([\w-]+)') or \
              match1(str(data), r'charset=\"([^\"]+)', 'charset=([^"]+)') or 'utf-8'
    try:
        data = data.decode(charset)
    except:
        from .log import w
        w("wrong charset for {}".format(url))
    return data

#DEPRECATED below, return None or 0
def url_size(url, faker = False):
    return 0

def urls_size(urls):
    return sum(map(url_size, urls))

def url_info(url, faker = False):
    return '', '', 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ykdl.util.html import get_content
from ykdl.util.match import match1
from ykdl.extractor import VideoExtractor

import json

class QQLive(VideoExtractor):
    name = u'QQ Live (企鹅直播)'

    mutli_bitrate = ['middle2', 'middle']

    bitrate_2_type = {'middle2': 'mid', 'middle': 'low'}

    def prepare(self):
        self.live = True
        if not self.vid:
            self.vid = match1(self.url, '/(\d+)')
        if not self.vid:
            html = get_content(self.url)
            self.vid = match1(html, '"room_id":(\d+)')

        #from upstream!!
        api_url = 'http://www.qie.tv/api/v1/room/{}'.format(self.vid)

        metadata = json.loads(get_content(api_url))
        assert metadata['error'] == 0, 'error {}: {}'.format(metadata['error'], metadata['data'])

        livedata = metadata['data']
        assert livedata['show_status'] == '1', 'error: live show is not on line!!'

        self.title = livedata['room_name']
        self.artist = livedata['nickname']

        base_url = livedata['rtmp_url']

        if 'hls_url' in livedata:
            self.stream_types.append('original')
            self.streams['original'] = {'container': 'm3u8', 'video_profile': 'original', 'src' : [livedata['hls_url']], 'size': float('inf')}

        mutli_stream = livedata['rtmp_multi_bitrate']
        for i in self.mutli_bitrate:
            if i in mutli_stream:
                self.stream_types.append(self.bitrate_2_type[i])
                self.streams[self.bitrate_2_type[i]] = {'container': 'flv', 'video_profile': self.bitrate_2_type[i], 'src' : [base_url + '/' + mutli_stream[i]], 'size': float('inf')}

site = QQLive()
            



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ykdl.util.match import match1
from ykdl.util.html import get_content
from ykdl.extractor import VideoExtractor

class Kuwo(VideoExtractor):
    name = u'KuWo (酷我音乐)'
    supported_stream_types = ['aac', 'mp3']
    def prepare(self):
        if not self.vid:
            self.vid = match1(self.url, 'yinyue/(\d+)')

        html = get_content("http://player.kuwo.cn/webmusic/st/getNewMuiseByRid?rid=MUSIC_{}".format(self.vid))
        self.title = match1(html, "<name>(.*)</name>")
        self.artist = match1(html, "<artist>(.*)</artist>")
        for t in self.supported_stream_types:
            url=get_content("http://antiserver.kuwo.cn/anti.s?format={}&rid=MUSIC_{}&type=convert_url&response=url".format(t, self.vid))

            self.stream_types.append(t)
            self.streams[t] = {'container': t, 'video_profile': 'current', 'src' : [url], 'size': 0}


site = Kuwo()



#!/usr/bin/env python

__all__ = ['mixcloud_download']

from ..common import *

def mixcloud_download(url, output_dir = '.', merge = True, info_only = False):
    html = get_html(url)
    title = r1(r'<meta property="og:title" content="([^"]*)"', html)
    preview_url = r1("m-preview=\"([^\"]+)\"", html)

    url = re.sub(r'previews', r'c/originals', preview_url)
    for i in range(10, 30):
        url = re.sub(r'stream[^.]*', r'stream' + str(i), url)

        try:
            type, ext, size = url_info(url)
            break
        except:
            continue

    try:
        type
    except:
        url = re.sub('c/originals', r'c/m4a/64', url)
        url = re.sub('.mp3', '.m4a', url)
        for i in range(10, 30):
            url = re.sub(r'stream[^.]*', r'stream' + str(i), url)

            try:
                type, ext, size = url_info(url)
                break
            except:
                continue

    print_info(site_info, title, type, size)
    if not info_only:
        download_urls([url], title, ext, size, output_dir)

site_info = "Mixcloud.com"
download = mixcloud_download
download_playlist = playlist_not_supported('mixcloud')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ykdl.simpleextractor import SimpleExtractor
from ykdl.util.html import get_content, add_header
from ykdl.util.match import match1, matchall

class HuyaLive(SimpleExtractor):
    name = u"Huya Live (虎牙直播)"

    def __init__(self):
        SimpleExtractor.__init__(self)
        add_header('User-Agent', 'Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334 Safari/7534.48.3')
        self.live = True
        self.title_pattern = 'liveRoomName = \"([^\"]+)'
        self.url_pattern = 'hasvedio: \'([^\']+)'
        self.artist_pattern = 'videoTitle = \"([^\"]+)'

    def get_info(self):
        return 'm3u8', float('inf')

site = HuyaLive()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..util.html import get_content, url_info
from ..extractor import VideoExtractor
import json

class SoundCloud(VideoExtractor):
    name = "SoundCloud"

    def prepare(self):

        if self.url and not self.vid:
            metadata = get_content('https://api.soundcloud.com/resolve.json?url=' + self.url + '&client_id=02gUJC0hH2ct1EGOcYXQIzRFU91c72Ea')
            info = json.loads(metadata)
            self.title = info["title"]
            self.vid = str(info["id"])
        else:
            self.title = self.name + "-" + self.vid

        url = 'https://api.soundcloud.com/tracks/' + self.vid + '/stream?client_id=02gUJC0hH2ct1EGOcYXQIzRFU91c72Ea'

        type, ext, size = url_info(url)

        self.streams['current'] = {'container': ext, 'src': [url], 'size' : size}
        self.stream_types.append('current')
 
site = SoundCloud()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re

def get_extractor(url):

    if re.search('my.tv.sohu.com', url):
        from . import my as s
        return s.site
    else:
        from . import tv as s
        return s.site
    raise NotImplementedError(url)
 



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..util.html import get_content
from ..util.match import match1, matchall
from ..extractor import VideoExtractor

from random import randint
import json

class Hunantv(VideoExtractor):
    name = u"芒果TV (HunanTV)"

    supported_stream_profile = [ u'超清', u'高清', u'标清' ]
    supported_stream_types = [ 'TD', 'HD', 'SD' ]
    profile_2_types = { u'超清': 'TD', u'高清': 'HD', u'标清': 'SD' }

    def prepare(self):

        if self.url and not self.vid:
            self.vid = match1(self.url, "/([0-9]+).html")

        rn = randint(0, 99999999)
        api_url = 'http://v.api.hunantv.com/player/video?video_id={}&random={}'.format(self.vid,rn)
        meta = json.loads(get_content(api_url))

        assert meta['status'] == 200, '[failed] status: {}, msg: {}'.format(meta['status'],meta['msg'])
        assert meta['data'], '[Failed] Video not found.'

        data = meta['data']

        info = data['info']
        self.title = info['title']
        for lstream in data['stream']:
            self.streams[self.profile_2_types[lstream['name']]] = {'container': 'm3u8', 'video_profile': lstream['name'], 'url' : lstream['url']}
            self.stream_types.append(self.profile_2_types[lstream['name']])
        self.stream_types= sorted(self.stream_types, key = self.supported_stream_types.index)

    def extract(self):
        if self.param.info:
            for stream in self.stream_types:
                meta = json.loads(get_content(self.streams[stream]['url']))
                self.streams[stream]['src'] = [meta['info']]
                self.streams[stream]['size'] = 0
            return
        else:
            stream_id = self.param.format or self.stream_types[0]

        meta = json.loads(get_content(self.streams[stream_id]['url']))
        self.streams[stream_id]['src'] = [meta['info']]
        self.streams[stream_id]['size'] = 0

    def download_playlist(self, url, param):
        self.url = url

        html = get_content(self.url, headers={})

        urls = matchall(html, ['"a-pic-play" href="([^"]+)"'])

        for url in urls:
            self.download(url, params)

site = Hunantv()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json
import hashlib
import base64

from ykdl.util.html import get_content, add_header
from ykdl.extractor import VideoExtractor
from ykdl.util.match import match1


def encrypted_id(dfsId):
    dfsId = str(dfsId)
    byte1 = bytearray('3go8&$8*3*3h0k(2)2', encoding='ascii')
    byte2 = bytearray(dfsId, encoding='ascii')
    byte1_len = len(byte1)
    for i in range(len(byte2)):
        byte2[i] = byte2[i] ^ byte1[i % byte1_len]
    m = hashlib.md5()
    m.update(byte2)
    result = base64.b64encode(m.digest()).decode('ascii')
    result = result.replace('/', '_')
    result = result.replace('+', '-')
    return result


def make_url(dfsId):
    encId = encrypted_id(dfsId)
    mp3_url = "http://m1.music.126.net/%s/%s.mp3" % (encId, dfsId)
    return mp3_url

class NeteaseMusic(VideoExtractor):
    name = u"Netease Music (网易云音乐)"

    supported_stream_types = ['hMusic', 'bMusic', 'mMusic', 'lMusic']
    song_date = {}
    def prepare(self):
        add_header("Referer", "http://music.163.com/")
        if not self.vid:
            self.vid =  match1(self.url, 'id=(.*)')

        api_url = "http://music.163.com/api/song/detail/?id={}&ids=[{}]&csrf_token=".format(self.vid, self.vid)
        music = json.loads(get_content(api_url))['songs'][0]

        self.title = music['name']
        self.artist = music['artists'][0]['name']

        for st in self.supported_stream_types:
            if st in music:
                self.stream_types.append(st)
                self.song_date[st] = music[st]
        
    def extract(self):
        stream_id = self.param.format or self.stream_types[0]
        song = self.song_date[stream_id]
        self.streams[stream_id] = {'container': song['extension'], 'video_profile': stream_id, 'src' : [make_url(song['dfsId'])], 'size': song['size']}

    def download_playlist(self, url, param):
        self.param = param
        add_header("Referer", "http://music.163.com/")
        vid =  match1(url, 'id=(.*)')
        if "album" in url:
           api_url = "http://music.163.com/api/album/{}?id={}&csrf_token=".format(vid, vid)
           listdata = json.loads(get_content(api_url))
           playlist = listdata['album']['songs']
        elif "playlist" in url:
           api_url = "http://music.163.com/api/playlist/detail?id={}&csrf_token=".format(vid)
           listdata = json.loads(get_content(api_url))
           playlist = listdata['result']['tracks']
        elif "toplist" in url:
           api_url = "http://music.163.com/api/playlist/detail?id={}&csrf_token=".format(vid)
           listdata = json.loads(get_content(api_url))
           playlist = listdata['result']['tracks']
        elif "artist" in url:
           api_url = "http://music.163.com/api/artist/{}?id={}&csrf_token=".format(vid, vid)
           listdata = json.loads(get_content(api_url))
           playlist = listdata['hotSongs']


        for music in playlist:
            self.title = music['name']
            self.artist = music['artists'][0]['name']
            for st in self.supported_stream_types:
                if st in music:
                    self.stream_types = []
                    self.stream_types.append(st)
                    self.song_date[st] = music[st]
            self.download_normal()

site = NeteaseMusic()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..util.match import match1
from ..util.html import get_content
from ..extractor import VideoExtractor


class Instagram(VideoExtractor):
    name = "Instagram"

    def prepare(self):

        if not self.url:
            self.url = 'instagram.com/p/{}'.format(self.vid)

        if not self.vid:
            self.vid = match1(self.url, 'instagram.com/p/([^/]+)')

        html = get_content(self.url)
        self.title = match1(html, '<meta property="og:title" content="([^"]*)"')
        stream = match1(html, '<meta property="og:video" content="([^"]*)"')
        mime, ext, size = url_info(stream)
        self.streams['current'] = {'container': ext, 'src': [stream], 'size' : size}
        self.stream_types.append('current')


site = Instagram()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .youkubase import YoukuBase
from ..util.html import get_content
from .youkujs import install_acode
import json

class Acorig(YoukuBase):
    name = u"AcFun 优酷合作视频"

    client_id = '908a519d032263f8'
    ct = 86

    def setup(self):

        if not self.title:
            self.title = self.name + "-" + self.vid

        self.vid = json.loads(get_content('http://www.acfun.tv/video/getVideo.aspx?id={}'.format(self.vid)))['sourceId']
        install_acode('v', 'b', '1z4i', '86rv', 'ogb', 'ail')
        self.get_custom_sign()
        self.get_custom_stream()


site = Acorig()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from importlib import import_module

from .util.match import match1
from .util.html import fake_headers
from .param import arg_parser
from .util import log

import socket
import os
import sys
from ykdl.compact import ProxyHandler, build_opener, install_opener

def mime_to_container(mime):
    mapping = {
        'video/3gpp': '3gp',
        'video/mp4': 'mp4',
        'video/webm': 'webm',
        'video/x-flv': 'flv',
    }
    if mime in mapping:
        return mapping[mime]
    else:
        return mime.split('/')[1]

alias = {
        '163': 'netease',
        'fun': 'funshion',
        'iask': 'sina',
        'in': 'alive',
        'kankanews': 'bilibili',
        'smgbb': 'bilibili',
        '7gogo': 'nanagogo'
}
def url_to_module(url):
    video_host = match1(url, 'https?://([^/]+)/')
    video_url = match1(url, 'https?://[^/]+(.*)')
    assert video_host and video_url, 'invalid url: ' + url

    if video_host.endswith('.com.cn'):
        video_host = video_host[:-3]
    domain = match1(video_host, '(\.[^.]+\.[^.]+)$') or video_host
    assert domain, 'unsupported url: ' + url

    k = match1(domain, '([^.]+)')
    if k in alias.keys():
        k = alias[k]
    try:
        m = import_module('.'.join(['ykdl','extractors', k]))
        if hasattr(m, "get_extractor"):
            site = m.get_extractor(url)
        else:
            site = m.site
        return site, url
    except(ImportError):
        from ykdl.compact import HTTPConnection
        conn = HTTPConnection(video_host)
        conn.request("HEAD", video_url, headers=fake_headers)
        res = conn.getresponse()
        location = res.getheader('location')
        if location is None:
            return import_module('ykdl.extractors.generalembed').site, url
        elif location != url:
            return url_to_module(location)
        else:
            raise ConnectionResetError(url)

def main():
    args = arg_parser()
    if args.timeout:
        socket.setdefaulttimeout(args.timeout)
    if args.proxy:
        proxy_handler = ProxyHandler({
            'http': args.proxy,
            'https': args.proxy
        })
        opener = build_opener(proxy_handler)
        install_opener(opener)

    #mkdir and cd to output dir
    if not args.output_dir == '.':
        if not os.path.exists(args.output_dir):
            try:
                os.mkdir(args.output_dir)
            except:
                log.w("No permission or Not found " + param_dict['output_dir'])
                log.w("use current folder")
                args.output_dir = '.'
    if os.path.exists(args.output_dir):
        os.chdir(args.output_dir)

    exit = 0
    for url in args.video_urls:
        try:
            m,u = url_to_module(url)
            if not u == url:
                args.video_urls[args.video_urls.index(url)]  = u
            if args.playlist:
                m.download_playlist(u, args)
            else:
                m.download(u, args)
        except AssertionError as e:
            log.wtf(str(e))
            exit = 1
        except RuntimeError as e:
            log.e(str(e))
            exit = 1
        except NotImplementedError as e:
            log.e(str(e))
            exit = 1
        except SyntaxError as e:
            log.e(str(e))
            exit = 1
    sys.exit(exit)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from importlib import import_module
from .common import alias

class EmbedExtractor():
    """
    this class is to help video embed site to handle
    video from other site.
    we just need to know the source site name, and video ID
    that's enough.
    with site name and VID, develop can easily to find out the real URL

    because embed site don't have video info, so they don't need stream_info.
    """

    def __init__(self):
        self.video_info = []
        self.title = None

    def prepare(self):
        """
        this API is to do real job to get site and VID
        sometimes title
        MUST override!!
        """
        pass

    def prepare_playlist(self):
        """
        this API is to do real job to get site and VID
        sometimes title
        MUST override!!
        """
        pass

    def download(self, url, param):
        self.param = param
        if isinstance(url, str) and url.startswith('http'):
            self.url = url
        self.video_info = []
        self.prepare()

        if not self.video_info:
            raise NotImplementedError(self.url + " is not supported")

        if self.video_info:
            n = 0
            for v in self.video_info:
                site, vid = v
                if site in alias.keys():
                    site = alias[site]
                s = import_module('.'.join(['ykdl','extractors',site])).site
                s.title = self.title + " %02d" % n
                s.download(vid, self.param)
                n += 1

    def download_playlist(self, url, param):
        self.param = param
        if isinstance(url, str) and url.startswith('http'):
            self.url = url
        self.video_info = []
        self.prepare_playlist()

        if not self.video_info:
            raise NotImplementedError('Playlist is not supported for ' + self.name)

        if self.video_info:
            for v in self.video_info:
                site, vid = v
                if site in alias.keys():
                    site = alias[site]
                s = import_module('.'.join(['ykdl','extractors',site])).site
                s.download_playlist(vid, self.param)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..util.html import get_content
from ..util.match import match1, matchall
from ..extractor import VideoExtractor
import json
import base64, hashlib, time

class Letvcloud(VideoExtractor):
    name = u"乐视云 (Letvcloud)"

    supported_stream_types = ['yuanhua', 'supper', 'high', 'low']

    def letvcloud_download_by_vu(self):
        #ran = float('0.' + str(random.randint(0, 9999999999999999))) # For ver 2.1
        #str2Hash = 'cfflashformatjsonran{ran}uu{uu}ver2.2vu{vu}bie^#@(%27eib58'.format(vu = vu, uu = uu, ran = ran)  #Magic!/ In ver 2.1
        vu, uu = self.vid
        argumet_dict ={'cf' : 'flash', 'format': 'json', 'ran': str(int(time.time())), 'uu': str(uu),'ver': '2.2', 'vu': str(vu), }
        sign_key = '2f9d6924b33a165a6d8b5d3d42f4f987'  #ALL YOUR BASE ARE BELONG TO US
        str2Hash = ''.join([i + argumet_dict[i] for i in sorted(argumet_dict)]) + sign_key
        sign = hashlib.md5(str2Hash.encode('utf-8')).hexdigest()
        html = get_content('http://api.letvcloud.com/gpc.php?' + '&'.join([i + '=' + argumet_dict[i] for i in argumet_dict]) + '&sign={sign}'.format(sign = sign), charset= 'utf-8')
        info = json.loads(html)

        available_stream_type = info['data']['video_info']['media'].keys()
        for stream in self.supported_stream_types:
            if stream in available_stream_type:
                urls = [base64.b64decode(info['data']['video_info']['media'][stream]['play_url']['main_url']).decode("utf-8")]
                size = urls_size(urls)
                ext = 'mp4'
                self.stream_types.append(stream)
                self.streams[stream] = {'container': ext, 'video_profile': stream, 'src': urls, 'size' : size}

    def prepare(self):

        if self.url and not self.vid:
            #maybe error!!
            self.vid = (vu, uu) = matchall(self.url, ["vu=([^&]+)","uu=([^&]+)"])
        _, vu = self.vid
        if not self.title:
            self.title = "LeTV - {}".format(vu)
        self.letvcloud_download_by_vu()

site = Letvcloud()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ykdl.extractor import VideoExtractor
from ykdl.util.html import get_content
from ykdl.util.match import match1, matchall


class BiliLive(VideoExtractor):
    name = u"Bilibili live (哔哩哔哩 直播)"

    def prepare(self):
        self.live = True
        if not self.vid:
            html = get_content(self.url)
            self.vid = match1(html, 'cid=([^&]+)')
            self.title = match1(html, '<title>([^<]+)')

        else:
            if not self.title:
                self.title = self.name + "-" + self.vid

        info = get_content('http://live.bilibili.com/api/playurl?cid={}'.format(self.vid))
        urls = [matchall(info, ['CDATA\[([^\]]+)'])[1]]
        size = float('inf')
        ext = 'flv'

        self.stream_types.append('current')
        self.streams['current'] = {'container': ext, 'video_profile': 'current', 'src' : urls, 'size': size}




site = BiliLive()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..util.html import get_content, url_info
from ..util.match import match1, matchall
from ..extractor import VideoExtractor

class Joy(VideoExtractor):

    name = u'激动网 (Joy)'


    def prepare(self):

        if not self.vid:
            self.vid = match1(self.url, 'resourceId=([0-9]+)')
        if not self.url:
            self.url = "http://www.joy.cn/video?resourceId={}".format(self.vid)

        html= get_content(self.url)

        self.title = match1(html, '<meta content=\"([^\"]+)')

        url = matchall(html, ['<source src=\"([^\"]+)'])[3]

        _, ext, size = url_info(url)

        self.stream_types.append('current')
        self.streams['current'] = {'container': ext, 'src': [url], 'size': size }

site = Joy()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re

def get_extractor(url):
    if re.search("music.douban", url):
        from . import music as s
        return s.site
    raise NotImplementedError(url)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .sohubase import SohuBase

class TvSohu(SohuBase):
    name = u'搜狐视频 (TvSohu)'

    apiurl = 'http://hot.vrs.sohu.com/vrs_flash.action?vid=%s'

site = TvSohu()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json

from ..util.match import match1
from ..util.html import get_content, add_header
from ..extractor import VideoExtractor
from ykdl.compact import compact_unquote


class Baomihua(VideoExtractor):

    name = u"爆米花（Baomihua)"

    def prepare(self):


        if self.url:
            self.vid = match1(self.url, '_(\d+)', 'm/(\d+)')

        add_header('Referer', 'http://m.video.baomihua.com/')
        html = get_content('http://play.baomihua.com/getvideourl.aspx?flvid={}&datatype=json&devicetype=wap'.format(self.vid))
        data = json.loads(html)
        self.title = compact_unquote(data["title"])
        host = data['host']
        stream_name = data['stream_name']
        t = data['videofiletype']
        size = int(data['videofilesize'])

        url = "http://{}/pomoho_video/{}.{}".format(host, stream_name, t)
        self.stream_types.append('current')
        self.streams['current'] = {'video_profile': 'current', 'container': t, 'src': [url], 'size' : size}

site = Baomihua()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ykdl.extractor import VideoExtractor
from ykdl.util.html import get_content
from ykdl.util.match import match1, matchall

import hashlib
import re

appkey='f3bb208b3d081dc8'

def parse_cid_playurl(xml):
    from xml.dom.minidom import parseString
    urls = []
    size = 0
    doc = parseString(xml.encode('utf-8'))
    for durl in doc.getElementsByTagName('durl'):
        urls.append(durl.getElementsByTagName('url')[0].firstChild.nodeValue)
        size += int(durl.getElementsByTagName('size')[0].firstChild.nodeValue)
    return urls, size

class BiliVideo(VideoExtractor):
    name = u'哔哩哔哩 (Bilibili)'
    supported_stream_profile = [u'超清', u'高清', u'流畅']
    profile_2_type = {u'超清': 'TD', u'高清': 'HD', u'流畅' :'SD'}
    def prepare(self):

        if not self.vid:
            html = get_content(self.url)
            self.vid = match1(html, 'cid=([^&]+)')
            self.title = match1(html, '<title>([^<]+)')
        for q in self.supported_stream_profile:
            api_url = 'http://interface.bilibili.com/playurl?appkey=' + appkey + '&cid=' + self.vid + '&quality=' + str(3-self.supported_stream_profile.index(q))
            urls, size = parse_cid_playurl(get_content(api_url))
            ext = 'flv'

            self.stream_types.append(self.profile_2_type[q])
            self.streams[self.profile_2_type[q]] = {'container': ext, 'video_profile': q, 'src' : urls, 'size': size}

    def prepare_list(self):
        html = get_content(self.url)
        video_list = matchall(html, ['<option value=\'([^\']*)\''])
        return ['http://www.bilibili.com'+v for v in video_list]

site = BiliVideo()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ykdl.util.html import get_content
from ykdl.util.match import match1, matchall
from ykdl.extractor import VideoExtractor

import xml.etree.ElementTree as ET

from ykdl.compact import urlencode, compact_bytes

import random
import base64
import struct
import uuid
import os

PLAYER_PLATFORM = 1
PLAYER_VERSION = '3.2.18.286'
"""
LEGACY FOR REFERENCE ONLY

def pack(data):
    target = []
    target.extend(struct.pack('>I', data[0]))
    target.extend(struct.pack('>I', data[1]))
    target = [c for c in target]
    return target

def unpack(data):
    data = ''.join([chr(b) for b in data])
    target = []
    data = data.encode('latin')
    target.extend(struct.unpack('>I', data[:4]))
    target.extend(struct.unpack('>I', data[4:8]))
    return target

def tea_encrypt(v, key):
    delta = 0x9e3779b9
    s = 0
    v = unpack(v)
    rounds = 16
    while rounds:
        s += delta
        s &= 0xffffffff
        v[0] += (v[1]+s) ^ ((v[1]>>5)+key[1]) ^ ((v[1]<<4)+key[0])
        v[0] &= 0xffffffff
        v[1] += (v[0]+s) ^ ((v[0]>>5)+key[3]) ^ ((v[0]<<4)+key[2])
        v[1] &= 0xffffffff
        rounds = rounds - 1
    return pack(v)

def qq_encrypt(data, key):
    temp = [0x00]*8
    enc = tea_encrypt(data, key)
    for i in range(8, len(data), 8):
        d1 = data[i:]
        for j in range(8):
            d1[j] = d1[j] ^ enc[i+j-8]
        d1 = tea_encrypt(d1, key)
        for j in range(len(d1)):
            d1[j] = d1[j]^data[i+j-8]^temp[j]
            enc.append(d1[j])
            temp[j] = enc[i+j-8]
    return enc

def strsum(data):
    s = 0
    for c in data:
        s = s*131 + ord(c)
    return 0x7fffffff & s

def ccc(platform, version, timestamp):
    key = [1735078436, 1281895718, 1815356193, 879325047]
    s1 = '537e6f0425c50d7a711f4af6af719e05d41d8cd98f00b204e9800998ecf8427e8afc2cf649f5c36c4fa3850ff01c1863d41d8cd98100b204e9810998ecf84271'
    d = [0x3039, 0x02]
    d.append(timestamp)
    d.append(platform)
    d.append(strsum(version))
    d.append(strsum(s1))
    data = [0xa6, 0xf1, 0xd9, 0x2a, 0x82, 0xc8, 0xd8, 0xfe, 0x43]
    for i in d:
        data.extend([c for c in struct.pack('>I', i)])
    data.extend([0x00]*7)
    enc = qq_encrypt(data, key)
    return base64.b64encode(bytes(enc), b'_-').replace(b'=', b'')

def load_key():
    url = 'http://vv.video.qq.com/checktime'
    tree = ET.fromstring(get_content(url))
    t = int(tree.find('./t').text)
    return ccc(PLAYER_PLATFORM, PLAYER_VERSION, t)

"""

def qq_get_final_url(url, fmt_name, type_name, br, form, fn):

    content = get_content('http://vv.video.qq.com/getkey',data=compact_bytes(form, 'utf-8'), charset = 'ignore')
    tree = ET.fromstring(content)

    vkey = tree.find('./key').text
    level = tree.find('./level').text
    sp = tree.find('./sp').text

    params = {
        'stdfrom': 'v1090',
        'type': type_name,
        'vkey': vkey,
        'level': level,
        'platform': PLAYER_PLATFORM,
        'br': br,
        'fmt': fmt_name,
        'sp': sp,
    }
    form = urlencode(params)
    return "%s?%s" % (url, form)



class QQ(VideoExtractor):

    name = u"腾讯视频 (QQ)"

    supported_stream_types = [ 'shd', 'mp4', 'hd', 'flv','sd' ]

    stream_2_profile = { 'shd': u'超清', 'mp4': u'高清mp4', 'hd': u'高清', 'flv': u'高清flv', 'sd': u'标清' }


    def get_stream_info(self, profile):


        player_pid = uuid.uuid4().hex.upper()

        params = {
            'fp2p': 1,
            'pid': player_pid,
            'otype': 'xml',
            'defn': profile,
            'platform': 1,
            'fhdswitch': 0,
            'charge': 0,
            'ckey' : "",
            'vid': self.vid,
            'defnpayver': 1,
            'encryptVer': "",
            'speed': random.randint(512, 1024),
            'ran': random.random(),
            'appver': PLAYER_VERSION,
            'defaultfmt': profile,
            'utype': -1,
            'vids': self.vid
        }

        form = urlencode(params)
        content = get_content('http://vv.video.qq.com/getinfo',data=compact_bytes(form, 'utf-8'), charset = 'ignore')
        tree = ET.fromstring(content)
        fmt_id = None
        fmt_name = None
        fmt_br = None
        for fmt in tree.findall('./fl/fi'):
            sl = int(fmt.find('./sl').text)
            if sl:
                fmt_id = fmt.find('./id').text
                fmt_name = fmt.find('./name').text
                fmt_br = fmt.find('./br').text

        video = tree.find('./vl/vi')
        filename = video.find('./fn').text
        self.title = video.find('./ti').text

        cdn = video.find('./ul/ui')
        cdn_url = cdn.find('./url').text
        filetype = int(cdn.find('./dt').text)
        vt = cdn.find('./vt').text
        if filetype == 1:
            type_name = 'flv'
        elif filetype == 2:
            type_name = 'mp4'
        else:
            type_name = 'unknown'

        num_clips = len(video.findall('./cl/ci'))
        size = int(video.find('./fs').text)

        fns = os.path.splitext(filename)

        #may have preformence issue when info_only
        urls =[]

        if num_clips == 0:
            params = {
                'ran': random.random(),
                'appver': PLAYER_VERSION,
                'otype': 'xml',
                'encryptVer': "",
                'platform': 1,
                'filename': filename,
                'vid': self.vid,
                'vt': vt,
                'charge': 0,
                'format': fmt_id,
                'cKey': "",
            }

            form = urlencode(params)
            clip_url = '%s%s' % (cdn_url, filename)
            urls.append(qq_get_final_url(clip_url, fmt_name, type_name, fmt_br, form, filename))

        else:
            for idx in range(1, num_clips+1):
                fn = '%s.%d%s' % (fns[0], idx, fns[1])
                params = {
                    'ran': random.random(),
                    'appver': PLAYER_VERSION,
                    'otype': 'xml',
                    'encryptVer': "",
                    'platform': 1,
                    'filename': fn,
                    'vid': self.vid,
                    'vt': vt,
                    'charge': 0,
                    'format': fmt_id,
                    'cKey': "",
                }
                form = urlencode(params)
                clip_url = '%s%s' % (cdn_url, fn)
                urls.append(qq_get_final_url(clip_url, fmt_name, type_name, fmt_br, form, fn))

        return fmt_name, type_name, urls, size

    def prepare(self):

        if not self.vid:
            html = get_content(self.url)
            self.vid = match1(self.url, 'vid=(\w+)') or match1(html, 'vid:\"([^\"]+)')


        for stream in self.supported_stream_types:
            fmt_name, type_name, urls, size = self.get_stream_info(stream)
            self.streams[fmt_name] = {'container': type_name, 'video_profile': self.stream_2_profile[fmt_name], 'src' : urls, 'size': size}
            if not fmt_name in self.stream_types:
                self.stream_types.append(fmt_name)
        self.stream_types = sorted(self.stream_types, key = self.supported_stream_types.index)

    def download_playlist(self, url, param):
        html = get_content(url)
        vids = matchall(html, ['id=\"([^\"]+)\S title'])
        for vid in vids:
            self.download(vid, param)

site = QQ()



#!/usr/bin/env python

try:
    import m3u8
    import time
    import signal
    stop = False
    def m3u8_live_stopper():
        default_INT_handle = None
        def handle(sig, x):
            print("stopping m3u8 live download!!")
            global stop
            stop = True
            if default_INT_handle:
                signal.signal(signal.SIGINT, default_INT_handle)

        default_INT_handle = signal.signal(signal.SIGINT, handle)

    def load_m3u8_playlist(url):
        stream_types = []
        streams = {}
        m = m3u8.load(url).playlists
        for l in m:
            stream_types.append(str(l.stream_info.bandwidth))
            streams[str(l.stream_info.bandwidth)] = {'container': 'm3u8', 'video_profile': str(l.stream_info.bandwidth), 'src' : [l.absolute_uri], 'size': 0}
        stream_types.sort()
        return stream_types, streams

    def load_m3u8(url):
        urls = []
        m =  m3u8.load(url)
        for seg in m.segments:
            urls.append(seg.absolute_uri)
        return urls

    def load_live_m3u8(url):
        """
        the stream is live stream. so we use sleep to simulate player. but not perfact!
        """
        m =  m3u8.load(url)
        now = d = 0
        i = 0
        m3u8_live_stopper()
        while True:
            if stop:
                print('stopped!!')
                raise StopIteration
            if i < len(m.segments):
                delta = d -( time.time() - now)
                if (delta) > 0:
                    time.sleep(delta)
                segurl = m.segments[i].absolute_uri
                now = time.time()
                d = m.segments[i].duration
                i += 1
                yield segurl
            else:
                i = 0
                delta = d -( time.time() - now)
                if (delta) > 0:
                    time.sleep(d - (time.time() - now))
                m = m3u8.load(url)
                now = time.time()
                d = 0
except:
    def load_m3u8_playlist(url):
        stream_types = ['current']
        streams['current'] = {'container': 'm3u8', 'video_profile': 'current', 'src' : [url], 'size': 0}

    def load_m3u8(url):
        return [url]

    def load_live_m3u8(url):
        return [url]




#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..util.html import *
from ..extractor import VideoExtractor

class Vk(VideoExtractor):
    name = 'VK'

    supported_stream_types = ['1080', '720', '480', '360', '240']

    def prepare(self):
        video_page = get_content(self.url)
        self.title = unescape_html(match1(video_page, '"title":"([^"]+)"'))
        info = dict(re.findall(r'\\"url(\d+)\\":\\"([^"]+)\\"', video_page))
        for quality in self.supported_stream_types:
            if quality in info:
                url = re.sub(r'\\\\\\/', r'/', info[quality])
                _, ext, size = url_info(url)
                self.stream_types.append(quality)
                self.streams[quality] = {'container': ext, 'video_profile': quality, 'src': [url], 'size': size }

site = Vk()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from distutils.core import setup

import os

def find_packages(*tops):
    packages = []
    for d in tops:
        for root, dirs, files in os.walk(d, followlinks=True):
            if '__init__.py' in files:
                packages.append(root)
    return packages

setup(
    name = "ykdl",
    version = "1.0.9.2",
    author = "Zhang Ning",
    author_email = "zhangn1985@gmail.com",
    url = "https://github.com/zhangn1985/ykdl",
    license = "MIT",
    description = "a video downloader written in Python",
    long_description = "a video downloader written in Python",
    keywords = "video download youtube youku",
    packages = find_packages('ykdl'),
    platforms = 'any',
    classifiers = [
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Topic :: Internet",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Multimedia",
        "Topic :: Multimedia :: Graphics",
        "Topic :: Multimedia :: Sound/Audio",
        "Topic :: Multimedia :: Video",
        "Topic :: Utilities"
    ],
    scripts = ['bin/ykdl']
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..extractor import VideoExtractor
from xml.dom.minidom import parseString
from ..util.html import get_content
from ..util.match import match1

class Ifeng(VideoExtractor):
    name = u'凤凰视频 (ifeng)'

    supported_stream_types = ['500k', '350k']

    def prepare(self):

        if not self.vid:
            self.vid= match1(self.url, '#([a-zA-Z0-9\-]+)', '/([a-zA-Z0-9\-]+).shtml')

        xml = get_content('http://v.ifeng.com/video_info_new/{}/{}/{}.xml'.format(self.vid[-2], self.vid[-2:], self.vid))
        doc = parseString(xml.encode('utf-8'))
        self.title = doc.getElementsByTagName('item')[0].getAttribute("Name")
        videos = doc.getElementsByTagName('videos')
        for v in videos[0].getElementsByTagName('video'):
            if v.getAttribute("mediaType") == 'mp4':
                _t = v.getAttribute("type")
                _u = v.getAttribute("VideoPlayUrl")
                self.stream_types.append(_t)
                self.streams[_t] = {'container': 'mp4', 'video_profile': _t, 'src' : [_u], 'size': 0}

        self.stream_types = sorted(self.stream_types, key = self.supported_stream_types.index)

site = Ifeng()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..util.html import get_content
from ..util.match import match1
from ..extractor import VideoExtractor

import json

class CNTV(VideoExtractor):
    name = u'央视网 (cctv)'

    supported_stream_types = ['normal', 'low']
    type_2_cpt = { 'normal':'chapters', 'low':'lowChapters' }

    def prepare(self):

        if self.url and not self.vid:
            content = get_content(self.url)
            self.vid = match1(content, 'guid = "([^"]+)')

        assert self.vid, 'cant find vid'

        html = get_content('http://vdn.apps.cntv.cn/api/getHttpVideoInfo.do?pid={}'.format(self.vid))
        data = json.loads(html)

        video_data = data['video']
        self.title = data['title']

        for t in self.supported_stream_types:
            if self.type_2_cpt[t] in video_data:
                urls = []
                for v in video_data[self.type_2_cpt[t]]:
                   urls.append(v['url'])
                self.stream_types.append(t)
                self.streams[t] = {'container': 'mp4', 'video_profile': t, 'src': urls, 'size' : 0}

site = CNTV()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ykdl.util.match import match1
from ykdl.util.html import get_content
from ykdl.extractor import VideoExtractor

import time
import json

class LongzhuLive(VideoExtractor):
    name = u'Longzhu Live (龙珠直播)'

    def prepare(self):
        self.live = True

        if not self.vid:
            html = get_content(self.url)
            self.vid = match1(html, 'roomid: (\d+)')
            self.title = match1(html, '"title":"([^"]+)')
            self.artist = match1(html, '"Name":"([^"]+)')

        api_url = 'http://livestream.plu.cn/live/getlivePlayurl?roomId={}&{}'.format(self.vid, int(time.time()))

        data = json.loads(get_content(api_url))['playLines'][0]['urls'] #don't know index 1

        for i in data:
            if i['ext'] == 'flv':
                self.stream_types.append("current")
                self.streams["current"] = {'container': 'flv', 'video_profile': 'current', 'src' : [i['securityUrl']], 'size': 0}

site = LongzhuLive()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .. import __name__ as library_name

import os, sys

IS_ANSI_TERMINAL = os.getenv('TERM') in (
    'eterm-color',
    'linux',
    'screen',
    'vt100',
    'xterm')

# ANSI escape code
# See <http://en.wikipedia.org/wiki/ANSI_escape_code>
RESET = 0
BOLD = 1
UNDERLINE = 4
NEGATIVE = 7
NO_BOLD = 21
NO_UNDERLINE = 24
POSITIVE = 27
BLACK = 30
RED = 31
GREEN = 32
YELLOW = 33
BLUE = 34
MAGENTA = 35
CYAN = 36
LIGHT_GRAY = 37
DEFAULT = 39
BLACK_BACKGROUND = 40
RED_BACKGROUND = 41
GREEN_BACKGROUND = 42
YELLOW_BACKGROUND = 43
BLUE_BACKGROUND = 44
MAGENTA_BACKGROUND = 45
CYAN_BACKGROUND = 46
LIGHT_GRAY_BACKGROUND = 47
DEFAULT_BACKGROUND = 49
DARK_GRAY = 90                 # xterm
LIGHT_RED = 91                 # xterm
LIGHT_GREEN = 92               # xterm
LIGHT_YELLOW = 93              # xterm
LIGHT_BLUE = 94                # xterm
LIGHT_MAGENTA = 95             # xterm
LIGHT_CYAN = 96                # xterm
WHITE = 97                     # xterm
DARK_GRAY_BACKGROUND = 100     # xterm
LIGHT_RED_BACKGROUND = 101     # xterm
LIGHT_GREEN_BACKGROUND = 102   # xterm
LIGHT_YELLOW_BACKGROUND = 103  # xterm
LIGHT_BLUE_BACKGROUND = 104    # xterm
LIGHT_MAGENTA_BACKGROUND = 105 # xterm
LIGHT_CYAN_BACKGROUND = 106    # xterm
WHITE_BACKGROUND = 107         # xterm

def sprint(text, *colors):
    """Format text with color or other effects into ANSI escaped string."""
    return "\33[{}m{content}\33[{}m".format(";".join([str(color) for color in colors]), RESET, content=text) if IS_ANSI_TERMINAL and colors else text

def println(text, *colors):
    """Print text to standard output."""
    sys.stdout.write(sprint(text, *colors) + "\n")

def print_err(text, *colors):
    """Print text to standard error."""
    sys.stderr.write(sprint(text, *colors) + "\n")

def print_log(text, *colors):
    """Print a log message to standard error."""
    sys.stderr.write(sprint("{}: {}".format(library_name, text), *colors) + "\n")

def i(message):
    """Print a normal log message."""
    print_log(message)

def d(message):
    """Print a debug log message."""
    print_log(message, BLUE)

def w(message):
    """Print a warning log message."""
    print_log(message, YELLOW)

def e(message):
    """Print an error log message."""
    print_log(message, YELLOW, BOLD)

def wtf(message):
    """What a Terrible Failure!"""
    print_log(message, RED, BOLD)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..util.html import get_content
from ..embedextractor import EmbedExtractor
from ..util.match import match1

import re

class Tudou(EmbedExtractor):
    name = "土豆 (tudou)"

    def prepare(self):

        if re.search('acfun', self.url):
            self.video_url.append(('acfun', self.url))
            return

        html = get_content(self.url)
        self.title = match1(html, '<title>([^<]+)')
        vcode = match1(html, 'vcode\s*[:=]\s*\'([^\']+)\'')
        if vcode:
            self.video_info.append(('youku', vcode))
        else:
            vid = match1(html, 'iid\s*[:=]\s*(\d+)')
            if vid:
                self.video_info.append(('tdorig', vid))

    def parse_plist(self):
        html = get_content(self.url)
        lcode = match1(html, "lcode:\s*'([^']+)'")
        plist_info = json.loads(get_content('http://www.tudou.com/crp/plist.action?lcode=' + lcode))
        return ([(item['kw'], item['iid']) for item in plist_info['items']])

    def download_playlist(self, url, param):
        exit(0)
        self.url = url
        videos = self.parse_plist()
        for i, (title, id) in enumerate(videos):
            print('Processing %s of %s videos...' % (i + 1, len(videos)))
            self.download(id, param)

site = Tudou()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ykdl.extractor import VideoExtractor
from ykdl.util.html import get_content
from ykdl.util.match import match1
from ykdl.compact import compact_str

import json

class HuyaVideo(VideoExtractor):
    name = u"huya video (虎牙视频)"

    supported_stream_types = ['BD', 'TD', 'HD', 'SD']

    stream_2_profile = {u'原画':"BD", u'超清': 'TD', u'高清': 'HD', u'流畅': 'SD' }

    def prepare(self):

        if not self.vid:
            self.vid = match1(self.url, 'play/(\d+)')

        if not self.vid:
            html = get_content(self.url)
            self.vid = match1(html, 'data-vid="(\d+)')

        api_url = 'http://playapi.v.duowan.com/index.php?vid={}&partner=&r=play%2Fvideo'.format(self.vid)
        data = json.loads(get_content(api_url))['result']['items']
        #lazy title
        self.title = self.name + '_' + self.vid

        for i in data:
            d = i['transcode']
            s = i['task_name'][0:2]
            p = self.stream_2_profile[compact_str(s)]
            self.stream_types.append(p)
            self.streams[p] = {'container': 'mp4', 'video_profile': s, 'src': [d['urls'][0]], 'size' : int(d['size'])}

        self.stream_types = sorted(self.stream_types, key = self.supported_stream_types.index)

site = HuyaVideo()



#!/usr/bin/env python

__all__ = ['pptv_download', 'pptv_download_by_id']

from ..common import *

import re
import time
import urllib
from random import random


def constructKey(arg):

    def str2hex(s):
        r=""
        for i in s[:8]:
            t=hex(ord(i))[2:]
            if len(t)==1:
                t="0"+t
            r+=t
        for i in range(16):
            r+=hex(int(15*random()))[2:]
        return r

    #ABANDONED  Because SERVER_KEY is static
    def getkey(s):
        #returns 1896220160
        l2=[i for i in s]
        l4=0
        l3=0
        while l4<len(l2):
            l5=l2[l4]
            l6=ord(l5)
            l7=l6<<((l4%4)*8)
            l3=l3^l7
            l4+=1
        return l3
        pass
    
    def rot(k,b): ##>>> in as3
        if k>=0:
            return k>>b
        elif k<0:
            return (2**32+k)>>b
        pass
     
    def lot(k,b):
        return (k<<b)%(2**32)

    #WTF?
    def encrypt(arg1,arg2):
        delta=2654435769
        l3=16;
        l4=getkey(arg2)  #1896220160
        l8=[i for i in arg1]
        l10=l4;
        l9=[i for i in arg2]
        l5=lot(l10,8)|rot(l10,24)#101056625
        # assert l5==101056625
        l6=lot(l10,16)|rot(l10,16)#100692230
        # assert 100692230==l6
        l7=lot(l10,24)|rot(l10,8)
        # assert 7407110==l7
        l11=""
        l12=0
        l13=ord(l8[l12])<<0
        l14=ord(l8[l12+1])<<8
        l15=ord(l8[l12+2])<<16
        l16=ord(l8[l12+3])<<24
        l17=ord(l8[l12+4])<<0
        l18=ord(l8[l12+5])<<8
        l19=ord(l8[l12+6])<<16
        l20=ord(l8[l12+7])<<24
     
        l21=(((0|l13)|l14)|l15)|l16
        l22=(((0|l17)|l18)|l19)|l20
     
        l23=0
        l24=0
        while l24<32:
            l23=(l23+delta)%(2**32)
            l33=(lot(l22,4)+l4)%(2**32)
            l34=(l22+l23)%(2**32)
            l35=(rot(l22,5)+l5)%(2**32)
            l36=(l33^l34)^l35
            l21=(l21+l36)%(2**32)
            l37=(lot(l21,4)+l6)%(2**32)
            l38=(l21+l23)%(2**32)
            l39=(rot(l21,5))%(2**32)
            l40=(l39+l7)%(2**32)
            l41=((l37^l38)%(2**32)^l40)%(2**32)
            l22=(l22+l41)%(2**32)
     
            l24+=1
     
        l11+=chr(rot(l21,0)&0xff)
        l11+=chr(rot(l21,8)&0xff)
        l11+=chr(rot(l21,16)&0xff)
        l11+=chr(rot(l21,24)&0xff)
        l11+=chr(rot(l22,0)&0xff)
        l11+=chr(rot(l22,8)&0xff)
        l11+=chr(rot(l22,16)&0xff)
        l11+=chr(rot(l22,24)&0xff)
     
        return l11        
            

    loc1=hex(int(arg))[2:]+(16-len(hex(int(arg))[2:]))*"\x00"
    SERVER_KEY="qqqqqww"+"\x00"*9
    res=encrypt(loc1,SERVER_KEY)
    return str2hex(res)


def pptv_download_by_id(id, title = None, output_dir = '.', merge = True, info_only = False):
    xml = get_html('http://web-play.pptv.com/webplay3-0-%s.xml?type=web.fpp' % id)
    #vt=3 means vod mode vt=5 means live mode
    host = r1(r'<sh>([^<>]+)</sh>', xml)
    k = r1(r'<key expire=[^<>]+>([^<>]+)</key>', xml)
    rid = r1(r'rid="([^"]+)"', xml)
    title = r1(r'nm="([^"]+)"', xml)

    st=r1(r'<st>([^<>]+)</st>',xml)[:-4]
    st=time.mktime(time.strptime(st))*1000-60*1000-time.time()*1000
    st+=time.time()*1000
    st=st/1000

    key=constructKey(st)

    pieces = re.findall('<sgm no="(\d+)"[^<>]+fs="(\d+)"', xml)
    numbers, fs = zip(*pieces)
    urls=[ "http://ccf.pptv.com/{}/{}?key={}&fpp.ver=1.3.0.4&k={}&type=web.fpp".format(i,rid,key,k)  for i in range(max(map(int,numbers))+1)]

    total_size = sum(map(int, fs))
    assert rid.endswith('.mp4')
    print_info(site_info, title, 'mp4', total_size)

    if not info_only:
        try:
            download_urls(urls, title, 'mp4', total_size, output_dir = output_dir, merge = merge)
        except urllib.error.HTTPError:
            #for key expired
            pptv_download_by_id(id, output_dir = output_dir, merge = merge, info_only = info_only)

def pptv_download(url, output_dir = '.', merge = True, info_only = False):
    assert re.match(r'http://v.pptv.com/show/(\w+)\.html$', url)
    html = get_html(url)
    id = r1(r'webcfg\s*=\s*{"id":\s*(\d+)', html)
    assert id
    pptv_download_by_id(id, output_dir = output_dir, merge = merge, info_only = info_only)

site_info = "PPTV.com"
download = pptv_download
download_playlist = playlist_not_supported('pptv')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..util.html import get_content
from ..util.match import match1
from ..extractor import VideoExtractor
from ykdl.compact import urlencode, compact_bytes

import json

class HuomaoTv(VideoExtractor):
    name = u'火猫 (Huomao)'

    supported_stream_types = ['BD', 'TD', 'HD', 'SD' ]

    stream_2_profile = {'BD': u"原画", 'TD': u'超清', 'HD': u'高清', 'SD': u'标清' }

    live_base = "http://www.huomaotv.cn/swf/live_data"

    def prepare(self):
        self.live = True
        html = get_content(self.url)
        self.title = match1(html, '<title>([^<]+)')

        video_name = match1(html, 'video_name = \'([^\']+)')
        params = { 'streamtype':'live',
                   'VideoIDS': video_name,
                   'cdns' : '1'
                }
        form = urlencode(params)
        content = get_content(self.live_base,data=compact_bytes(form, 'utf-8'),charset = 'utf-8')
        stream_data = json.loads(content)
        if stream_data["roomStatus"] == "1":
            for stream in stream_data["streamList"]:
                if stream['default'] == 1:
                    defstream = stream['list']
        else:
           from ..util import log
           log.e("The live stream is not online! ")
           exit(1)
        for stream in defstream:
            self.stream_types.append(stream['type'])
            self.streams[stream['type']] = {'container': 'flv', 'video_profile': self.stream_2_profile[stream['type']], 'src' : [stream['url']], 'size': float('inf')}

        self.stream_types = sorted(self.stream_types, key = self.supported_stream_types.index)

site = HuomaoTv()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..util.html import get_content
from ..util.match import match1, matchall
from ..embedextractor import EmbedExtractor

class Acfun(EmbedExtractor):

    def prepare(self):

        html = get_content(self.url)

        self.title = match1(html, '<h1 id="txt-title-view">([^<>]+)<')

        sourceType = match1(html, "data-from=\"([a-zA-Z0-9]+)\" data-did=")
        sourceId = match1(html, "data-did=\"\" data-sid=\"([a-zA-Z0-9=]+)\"")
        sourceVid = match1(html, "data-vid=\"([a-zA-Z0-9=]+)\" data-scode=")

        if sourceType == 'zhuzhan':
            sourceType = 'acorig'
            sourceId = sourceVid
        elif sourceType == 'letv':
            #workaround for letv, because it is letvcloud
            sourceType = 'letvcloud'
            sourceId = (sourceId, '2d8c027396')

        self.video_info.append((sourceType, sourceId))

    def download_playlist(self, url, param):
        self.url = url

        html = get_content(self.url)

        videos = matchall(html, ['href="([\/a-zA-Z0-9_]+)" title="Part'])

        for v in videos:
            next_url = "http://www.acfun.tv/{}".format(v)
            self.download(next_url, param)

site = Acfun()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..simpleextractor import SimpleExtractor

class Nanagogo(SimpleExtractor):
    name = '7gogo'

    def __init__(self):
        SimpleExtractor.__init__(self)
        self.title_pattern = '<meta property="og:title" content="([^"]+)"'
        self.url_pattern = '<meta property="og:video" content="([^"]*)"'

site=Nanagogo()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ..util.html import get_content
from ..util.match import match1, matchall
from ..extractor import VideoExtractor

import hashlib
import time
import json

douyu_match_pattern = [ 'class="hroom_id" value="([^"]+)',
                        'data-room_id="([^"]+)'
                      ]
class Douyutv(VideoExtractor):
    name = u'斗鱼 (DouyuTV)'

    def prepare(self):
        self.live = True
        if self.url:
            self.vid = self.url[self.url.rfind('/')+1:]

        suffix = 'room/%s?aid=android&client_sys=android&time=%d' % (self.vid, int(time.time()))
        sign = hashlib.md5((suffix + '1231').encode('ascii')).hexdigest()
        json_request_url = "http://www.douyutv.com/api/v1/%s&auth=%s" % (suffix, sign)
        content = get_content(json_request_url)
        data = json.loads(content)['data']
        server_status = data.get('error',0)
        if server_status is not 0:
            raise ValueError("Server returned error:%s" % server_status)
        self.title = data.get('room_name')
        show_status = data.get('show_status')
        assert show_status == "1", "The live stream is not online! (Errno:%s)" % show_status
        real_url = data.get('rtmp_url')+'/'+data.get('rtmp_live')
        self.stream_types.append('current')
        self.streams['current'] = {'container': 'flv', 'video_profile': 'current', 'src' : [real_url], 'size': float('inf')}

    def download_playlist(self, url, param):
        self.url = url
        self.param = param
        html = get_content(self.url)
        vids = matchall(html, douyu_match_pattern)
        for vid in vids:
            self.download(vid, param)

site = Douyutv()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re

def match1(text, *patterns):
    """Scans through a string for substrings matched some patterns (first-subgroups only).

    Args:
        text: A string to be scanned.
        patterns: Arbitrary number of regex patterns.

    Returns:
        When matches, returns first-subgroups from first match.
        When no matches, return None
    """

    for pattern in patterns:
        try:
            match = re.search(pattern, text)
        except(TypeError):
            match = re.search(pattern, str(text))
        if match:
            return match.group(1)
    return None


def matchall(text, patterns):
    """Scans through a string for substrings matched some patterns.

    Args:
        text: A string to be scanned.
        patterns: a list of regex pattern.

    Returns:
        a list if matched. empty if not.
    """

    ret = []
    for pattern in patterns:
        try:
            match = re.findall(pattern, text)
        except(TypeError):
            match = re.findall(pattern, str(text))
        ret += match

    return ret
